package GraphGenerator;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.security.SecureRandom;

public class Generator {
	public static void main(String[] args) throws FileNotFoundException {
		OutputStream out = System.out;
		int size = -1;
		int faults = -1;
		for (int i = 0; i < args.length; i++) {
			if (args[i].startsWith("-path="))
				out = new FileOutputStream(new File(args[i].substring(6)));
			else if (args[i].startsWith("-size="))
				size = Integer.parseInt(args[i].substring(6));
			else if (args[i].startsWith("-faults="))
				faults = Integer.parseInt(args[i].substring(8));
		}
		BufferedWriter outWriter = new BufferedWriter(new OutputStreamWriter(
				out));

		if (size == -1)
			System.out.println("Argument size missing, Usage: java -jar tourGen.jar -size=10 -faults=10");
		else if (faults == -1)
			System.out.println("Argument size missing, Usage: java -jar tourGen.jar -size=10 -faults=10");
		else
			generateGraph(outWriter, size, faults);

		try {
			outWriter.close();
		} catch (IOException e) {
		}
	}

	public static void generateGraph(Writer outWriter, int size, int distortions)
	{
		try {
			generateGraph_internal(outWriter, size, distortions);
		} catch (IOException e) {
			System.out.println("Error while writing tournament");
		}
	}

	private static void generateGraph_internal(Writer outWriter, int size, int edgesToAdd) throws IOException
	{
		SecureRandom rand = new SecureRandom();

		boolean[][] hasEdge = new boolean[size][size];
		
		int nextNodeNum = 1;
		//generate tree first
		while(nextNodeNum < size)
		{
			int connector = rand.nextInt(nextNodeNum);
			hasEdge[nextNodeNum][connector] = hasEdge[connector][nextNodeNum] = true;
			outWriter.write("V_" + connector + " " + "V_" + nextNodeNum+"\n");
			nextNodeNum++;
		}
		for (int i = 0; i < edgesToAdd; i++) {
			while(true)
			{
				int x = rand.nextInt(size);
				int y = rand.nextInt(size);
				if(x != y && !hasEdge[x][y])
				{
					hasEdge[x][y] = hasEdge[y][x] = true;
					outWriter.write("V_" + x + " " + "V_" + y+"\n");
					break;
				}
			}
		}
	}
}
