package graphLoading;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import main.Graph;

public class GraphLoader {
	private static boolean readGraphData(BufferedReader reader, HashMap<Integer, String> indexToName, ArrayList<IntBasedEdge> edgeList)
	{
		HashMap<String, Integer> nameToIndex = new HashMap<String, Integer>();
		try {
			String s;
			int i = 0;
			while((s = reader.readLine()) != null)
			{
				s = s.trim();
				if(s.startsWith("#"))
					continue;
				String[] endNodes = s.split(" ");
				if(endNodes.length != 2)
				{
					continue;
				}
				if(!nameToIndex.containsKey(endNodes[0])) {
					nameToIndex.put(endNodes[0], i);
					indexToName.put(i, endNodes[0]);
					i++;
				}
				if(!nameToIndex.containsKey(endNodes[1])) {
					nameToIndex.put(endNodes[1], i);
					indexToName.put(i, endNodes[1]);
					i++;
				}

				IntBasedEdge e = new IntBasedEdge(nameToIndex.get(endNodes[0]), nameToIndex.get(endNodes[1]));
				edgeList.add(e);
			}
			reader.close();
			return true;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
	}


	public static Graph[] loadGraph(BufferedReader reader, boolean loadSeparated) {
		HashMap<Integer, String> indexToName = new HashMap<Integer, String>();
		ArrayList<IntBasedEdge> edges = new ArrayList<IntBasedEdge>();

		readGraphData(reader, indexToName, edges);

		ArrayList<GraphComponent> componentList = new ArrayList<GraphComponent>();
		for(IntBasedEdge e : edges){
			GraphComponent currentComp = null;
			for(int i = 0; i < componentList.size(); i++ )
			{
				GraphComponent comp = componentList.get(i);
				if(comp.isConnected(e) || !loadSeparated)
				{
					if(currentComp == null)
					{
						currentComp = comp;
						currentComp.addEdge(e);
					}
					else
					{
						currentComp.addAll(comp);
						componentList.remove(i);
						i--;
					}
				}
			}
			if(currentComp == null)
			{
				currentComp = new GraphComponent();
				currentComp.addEdge(e);
				componentList.add(currentComp);
			}
		}
		Graph[] graphs = new Graph[componentList.size()];
		for(int i = 0; i < graphs.length; i++){
			GraphComponent comp = componentList.get(i);

			HashMap<Integer,Integer> oldToNewIndex = new HashMap<Integer, Integer>();
			String[] names = new String[comp.nodeList.size()];
			for(int n = 0; n < comp.nodeList.size(); n++)
			{
				int oldIndex = comp.nodeList.get(n);
				names[n] = indexToName.get(oldIndex);
				oldToNewIndex.put(oldIndex, n);
			}
			ArrayList<IntBasedEdge> edgeList = new ArrayList<IntBasedEdge>(comp.edgeList.size());
			for(IntBasedEdge e : comp.edgeList)
				edgeList.add(new IntBasedEdge(oldToNewIndex.get(e.left), oldToNewIndex.get(e.right)));

			graphs[i] = new Graph(names, edgeList);
		}
		return graphs;
	}
}
