package main;

import java.awt.IllegalComponentStateException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

import reductionRules.path.PathRedLPYSet;
import visualization.Graph.GraphViewing;
import Utils.ExpandMode;
import Utils.GraphUtils;

/**
 * represents a special case of a Degree2Path where it is a single cycle not connected to any other cyclic Neighbour
 * so the first and the last nodes are inner path nodes as well (but they are not identical)
 * 
 * @author Ich
 */
public class Degree2Circle extends Degree2Path {

	public Degree2Circle(ArrayList<Node> innerPathNodes, Graph graph) {
		super(innerPathNodes, graph);
	}

	@Override
	public boolean isSimpleCircle()
	{
		return true;
	}
	
	@Override
	public Node getPathVertex(int vertexNum)
	{
		return getInnerPathVertex(vertexNum);
	}
	
	@Override
	public Node getInnerPathVertex(int vertexNum)
	{
		if(vertexNum < 0)
			vertexNum += pathNodes.size();
		return pathNodes.get(vertexNum % pathNodes.size());
	}

	@Override
	public void removeNodesFromPath(int first, int last)
	{
		Node firstNode = pathNodes.get(first);
		Node firstNode_1 = getInnerPathVertex(first-1+pathNodes.size());

		Node lastNode = pathNodes.get(last);
		Node lastNode1 = getInnerPathVertex(last+1);

		GraphUtils.removeEdge(firstNode_1, firstNode);
		GraphUtils.removeEdge(lastNode, lastNode1);
		GraphUtils.addEdge(firstNode_1, lastNode1);

		if(first == last)
			pathNodes.remove(lastNode);
		else
		{
			Iterator<Node> pathIterator = pathNodes.iterator();
			int count = 0;
			while(pathIterator.hasNext())
			{
				pathIterator.next();
				if(last < first)
				{
					if(count <= last || count >= first)
						pathIterator.remove();
				}
				else
				{
					if(count > last)
						break;
					if(count >= first)
						pathIterator.remove();
				}
				count++;
			}
		}
		if(pathNodes.size() == 3)
			this.graph.addThreeCycle(getPathVertex(0), getPathVertex(1), getPathVertex(2));
		
		if(tokens != null && tokens.length>0)//due to removal of nodes some tokens might be invalid
			findAndSetTokens();
	}

	@Override
	public HashSet<Node> getPendantTreeNodes(int first, int last)
	{
		HashSet<Node> result = new HashSet<Node>();

		if(first == last)
			return GraphUtils.expand(pathNodes.get(first), ExpandMode.nonCyclic);
		
		Iterator<Node> pathIterator = pathNodes.iterator();
		int count = 0;
		
		while(pathIterator.hasNext())
		{
			Node n = pathIterator.next();
			if(last < first)
			{
				if(count <= last || count >= first)
					result.addAll(GraphUtils.expand(n, ExpandMode.nonCyclic));
			}
			else
			{
				if(count > last)
					break;
				if(count >= first)
					result.addAll(GraphUtils.expand(n, ExpandMode.nonCyclic));
			}
			count++;
		}
		
		return result;
	}

	@Override
	public void findAndSetTokens()
	{
		ArrayList<Token> result = new ArrayList<Token>();
		for(int i = 0; i < pathNodes.size(); i++)
		{
			Node n = pathNodes.get(i);
			switch(PathRedLPYSet.getP2Count(n))
			{
			case 0:
				break;
			case 1: 
				result.add(new Token(this, concatArrays(getLowerToken(i), getUpperToken(i)), i));
				break;
			case 2: 
				result.add(new Token(this, getLowerToken(i), i));
				result.add(new Token(this, getUpperToken(i), i));
				break;
			default:
				{
					GraphViewing.showGraphAndWaitForClose(graph, "tokenFinder");
					throw new IllegalComponentStateException();
				}
			}
		}
		tokens = result.toArray(new Token[0]);
	}
	
	@Override
	protected int[] getUpperToken(int node)
	{
		int node1 = (node+1)%pathNodes.size();
		int node2 = (node+1)%pathNodes.size();

		if(pathNodes.get(node1).getNeighbourCount() == 2)//if crossable
			return new int[]{node1, node2};
		else
			return new int[]{node1};
	}

	@Override
	protected int[] getLowerToken(int node)
	{
		int node_1 = (node-1+pathNodes.size())%pathNodes.size();
		if(pathNodes.get(node_1).getNeighbourCount() == 2)//if crossable
			return new int[]{node_1, node};
		else
			return new int[]{node};
	}



}
