package main;
import graphLoading.GraphLoader;

import java.awt.IllegalComponentStateException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;

import javax.swing.plaf.basic.BasicInternalFrameTitlePane.RestoreAction;

import Utils.GraphUtils;
import Utils.Tuple;

import reductionRules.Reduction;
import result.ResultList;
import solving.BranchSolver;
import solving.BranchSolverStep;
import solving.GraphCopy;
import visualization.Graph.GraphViewing;

public class Start 
{
	public static boolean useInitReductionVisualization = false;//currently not working
	public static boolean useBranchTreeVisualization = false;//currently not working

	//enable checks for consistency
	public static boolean debugMode = false;
	
	//check every solution for correctness
	public static boolean verifySolution = false;
	/**
	 * start the program
	 * uses the input stream system.in for input if available, else uses the first argument
	 * the next argument is optionally a boolean that determines whether the visualization of the algorithms is activated
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		int nextarg = -1;
		BufferedReader reader = null;
		if(System.in.available() > 0)
		{
			reader = new BufferedReader(new InputStreamReader(System.in));
			nextarg = 0;
		}
		else
		{
			reader =  new BufferedReader(new FileReader(args[0]));
			nextarg = 1;
		}

		if(args.length > nextarg)
			useInitReductionVisualization = Boolean.parseBoolean(args[nextarg]);

		Graph[] graphs = GraphLoader.loadGraph(reader, true);

		reader.close();
		
		BranchSolverStep.recsteps = 0;
		
		ResultList resultList = new ResultList();
		
		int k = 0;
		int initialKReduction = 0;
		int nodesBefore = 0;
		int nodesAfter = 0;
		int edgesCount = 0;
		for(Graph g : graphs)
		{
//			GraphViewing.showGraphAndWaitForClose(g, "");

			edgesCount += g.initialEdgesCount;
			nodesBefore+=g.nodes.size();

			g.validateConsistency();

			g.currentState = SolvingState.TRR;
			initialKReduction += Reduction.applyTreeReductionRules(g, resultList);

			g.validateConsistency();
			
//			GraphViewing.showGraphAndWaitForClose(g, "after Tree Reduction");

			g.currentState = SolvingState.PRR;
			initialKReduction += Reduction.applyPathReductionRules(g, resultList);

//			GraphViewing.showGraphAndWaitForClose(g, "after Path Reduction");
			
			Reduction.removeCaterpillars(g);

			g.currentState = SolvingState.BRANCHING;
			//System.out.println(BranchSolver.applyOldSolver(g, 10));
			k += BranchSolverStep.startBranching(g, resultList);

			nodesAfter += g.nodes.size();

			//for(Node n : g.nodes)
			//	System.out.println(n.name+" "+n.isCyclic);
			//System.out.println("Start: "+ nodesBefore);
			//System.out.println("Tree: "+ nodesAfterTree);
			//System.out.println("Path: "+ nodesAfterPath);

		}
		k += initialKReduction;
		System.out.println("#k "+k);
		//System.out.println("#dn "+(nodesBefore-nodesAfter));
		System.out.println("#f "+(edgesCount-nodesBefore+graphs.length));

		System.out.println("#reduced "+resultList.getReducedCount());
		System.out.println("#recsteps "+BranchSolverStep.recsteps);
		
		for(String edge : resultList)
			System.out.println(edge);
		if(resultList.size() != k)
			throw new IllegalComponentStateException();

		if(verifySolution)
		{
			reader =  new BufferedReader(new FileReader(args[0]));
			Graph[] g2 = GraphLoader.loadGraph(reader, false);
			checkSolution(g2[0], resultList);
		}
	}

	private static void checkSolution(Graph g, ResultList resultList)
	{
		GraphCopy copy = new GraphCopy(g);
		HashSet<Tuple<String,String>> deletedEdges = resultList.getFinalEdgesSet();
		HashMap<String, Node> nameToNode = new HashMap<String, Node>();
		for(Node n : g.nodes)
		{
			nameToNode.put(n.name, n);
		}
		for(Tuple<String, String> edge : deletedEdges)
		{
			Node n1 = nameToNode.get(edge.left);
			Node n2 = nameToNode.get(edge.right);
			if(n1 == null || n2 == null || !GraphUtils.hasEdge(n1, n2))
				throw new IllegalComponentStateException();
			GraphUtils.removeEdge(n1, n2);
		}
		g.markCyclicNodes();
		Reduction.removeCaterpillars(g);
		if(g.nodes.size()>0)
		{
			GraphViewing.showGraphAndWaitForClose(copy, "original");
			GraphViewing.showGraphAndWaitForClose(g, "remains after removing result edges and caterpillars");
			throw new IllegalComponentStateException();
		}
	}
}
