package reductionRules;

import java.util.HashSet;

import main.Graph;
import main.Node;
import visualization.GraphChanger;
import Utils.GraphUtils;

public abstract class ReductionRule extends GraphChanger 
{

	/**
	 * deletes a node from the graph
	 * @param g the graph containing the node
	 * @param n the node to delete
	 * @param removeFromNeighbourLists true if the node should be remove from it's neighbours neighbourlists as well, not nescessary if the neighbours are about to removed too or if the edges will be removed separately
	 */
	protected void deleteNode(Graph g, Node n, boolean removeFromNeighbourLists)
	{
		n.deleted = true;
		g.nodes.remove(n);
		onNodeDeleted(n);
		if(removeFromNeighbourLists)
			n.removeFromNeighbourLists();
	}

	protected void deleteNodes(Graph g, HashSet<Node> nodes, boolean removeFromNeighbourLists)
	{
		for(Node n : nodes)
		{
			if(removeFromNeighbourLists)
				n.removeFromNeighbourLists();
			n.deleted = true;
		}
		g.nodes.removeAll(nodes);
		onNodesDeleted(nodes);
	}

	/**
	 * Adds an edge between the two given nodes.
	 * @param n1
	 * @param n2
	 */
	protected void addEdge(Node n1, Node n2)
	{
		onEdgeAdded(n1, n2);
		GraphUtils.addEdge(n1, n2);
	}

	/**
	 * Deletes an edge between the two given nodes.
	 * @param n1
	 * @param n2
	 */
	protected void deleteEdge(Node n1, Node n2)
	{
		onEdgeDeleted(n1, n2);
		GraphUtils.removeEdge(n1, n2);
	}
}
