package reductionRules.path;

import java.util.ArrayList;

import Utils.Tuple;

import result.ResultList;
import main.Graph;
import main.Node;

/**
 * Path Reduction Rule 1:
 * Let P denote a degree-2-path and let both v(i) and v(i+1) be inner path vertices of P such that Y=T^v(i) is a Y-graph. If
 * 1. T^v(i+1) is neither a singleton nor a Y-graph, or
 * 2. v(i+2) is an inner path vertex, deg(v(i+1))=2, and T^v(i+2) is either a singleton or a Y-graph
 * then delete the edge {v(i), c(Y)} and decrease k by one. 
 * @author Steffen Toborg
 */
public class PRR1_CutOffYGraph extends PRR_RemoveYGraph 
{
	@Override	
	protected boolean applyRule(Graph g, Node ni_1, Node ni, Node ni1, Node ni2, ResultList resultList)
	{
		if((!ni1.isSingleton() && !PathRedLPYSet.hasYGraph(ni1))
				|| (ni2 != null && ni1.isSingleton() && (!ni2.isSingleton() || PathRedLPYSet.hasYGraph(ni2))))
//		if((ni1.L.size() == 0 && ni1.Y.size() == 0)
//				|| (ni2 != null && ni1.getNeighbourCount() == 2 && (ni2.L.size() != 0 || ni2.Y.size() != 0)))
		{
			Node ny = deleteYGraph(g, ni, true);

			if(ni.nonCyclicNeighbours.size() == 0)//y graph already removed
			{
				ArrayList<Tuple<Node, Node>> del = new ArrayList<Tuple<Node, Node>>();
				ArrayList<Tuple<Node, Node>> notDel = new ArrayList<Tuple<Node, Node>>();
				del.add(new Tuple<Node, Node>(ni_1, ni));
				del.add(new Tuple<Node, Node>(ni, ni1));
				notDel.add(new Tuple<Node, Node>(ni, ny));
				resultList.addEdgeReplacer(true, ni_1, ni, ni, ni1, del, notDel);
			}
			else
			resultList.add(true, ni, ny);
			
			return true;
		}
		return false;
	}
	
	
	@Override
	protected boolean deletesConnectingPoint()
	{
		return false;
	}

	@Override
	public String getName() {
		return "Path-Reduction-Rule 1: Cut off Y-graph";
	}
}
