package reductionRules.path;

import java.util.ArrayList;

import result.ResultList;

import Utils.GraphUtils;
import Utils.Tuple;

import main.Graph;
import main.Node;

/**
 * Path Reduction Rule 2:
 * Let P denote a degree-2-path and let both v(i) and v(i+1) be inner path vertices of P such that Y=T^v(i) is a Y-graph. If
 * 1. T^v(i+1) is a Y-graph, or
 * 2. v(i-1) and v(i+1) have degree two, or
 * 3. v(i+2) is an inner path vertex, deg(v(i+1))=2, and T^v(i+2) is neither a singleton nor a Y-graph
 * then delete the edge {v(i), c(Y)} and decrease k by one. 
 * @author Steffen Toborg
 */
public class PRR2_CutOutYGraph extends PRR_RemoveYGraph
{
	@Override
	protected boolean applyRule(Graph g, Node ni_1, Node ni, Node ni1, Node ni2,  ResultList resultList) {
		//		if(ni1.Y.size() == 1 //condition 1
		//				|| (ni1.getNeighbourCount() == 2 && ni_1.getNeighbourCount() == 2) //condition 2
		//				|| (ni2 != null  && ni1.getNeighbourCount() == 2 && ni2.L.size() == 0 && ni2.Y.size() == 0))//condition 3
		if(PathRedLPYSet.hasYGraph(ni1) //condition 1
				|| (ni1.isSingleton() && ni_1.getNeighbourCount() == 2) //condition 2
				|| (ni2 != null  && ni1.isSingleton() && !ni2.isSingleton() && !PathRedLPYSet.hasYGraph(ni2)))//condition 3
		{
			//Do not apply for paths that form a threecycle
			if(GraphUtils.hasEdge(ni_1, ni1))
				return false;
			
			//ni gets deleted anyway, no need to destroy any edge
			Node ny = deleteYGraph(g, ni, false);

			Node leaf = PathRedLPYSet.getLeaf(ni);
			if(leaf != null)
				deleteNode(g, leaf, false);

			deleteNode(g, ni, true);
			addEdge(ni1, ni_1);
			
			ArrayList<Tuple<Node, Node>> eDelEdges = new ArrayList<Tuple<Node,Node>>(2);
			eDelEdges.add(new Tuple<Node, Node>(ni_1, ni));
			eDelEdges.add(new Tuple<Node, Node>(ni, ni1));
			resultList.addEdgeReplacer(true, ni1, ni_1, eDelEdges, ni, ny);
			return true;
		}
		return false;
	}

	@Override
	protected boolean deletesConnectingPoint()
	{
		return true;
	}

	@Override
	public String getName() {
		return "Path-Reduction-Rule 2: Cut out Y-graph";
	}
}
