package reductionRules.path;

import result.ResultList;
import Utils.GraphUtils;
import main.Degree2Path;
import main.Graph;
import main.Node;

/**
 * Path Reduction Rule 3:
 * If there is a degree-2-path P with |V(T_P)| > 7 and K(P)={} (it has no tokens), then contract T_2,l-1 to a single vertex. 
 * @author Steffen Toborg
 */
public class PRR3_ContractPath extends PathReductionRule
{
	@Override
	protected int applyRule(Graph g, Degree2Path path, ResultList resultList) {

		//assume that the biggest subtree is at most a leaf if there are no tokens
		if(path.tokens.length == 0)
		{
			boolean isCircle = path.isSimpleCircle();
			int reducableNodes = 0;
			for(int i = isCircle? 1 : 2 ; i < path.pathNodes.size()-2; i++)
			{
				Node n = path.pathNodes.get(i);
				reducableNodes += 1+n.nonCyclicNeighbours.size();
			}
			if(reducableNodes < 2)
				return 0;

			int removeStart = isCircle? 2 : 3;
			int removeEnd = path.pathNodes.size()-3;

			//HashSet<Node> toRemove = path.getPendantTreeNodes(removeStart, removeEnd);
			Node remainingNode = path.getPathVertex(removeStart-1);
			Node leaf = PathRedLPYSet.getLeaf(remainingNode);
			if(leaf != null)
			{
				GraphUtils.removeEdge(leaf, remainingNode);
				deleteNode(g, leaf, false);
			}
			Node afterLastRemoveNode = path.getPathVertex(removeEnd+1);
			/*Node lastRemoveNode = path.getPathVertex(removeEnd);
			Node afterLastRemoveNode = path.getPathVertex(removeEnd+1);
			remainingNode.addCustomName(path.getPathVertex(removeStart).name.getName(remainingNode), afterLastRemoveNode);
			lastNode1.addCustomName(lastNode.name.getName(lastNode), firstNode_1);
			 */
			
			if(removeStart <= removeEnd)//we do not always need to remove an innerPath vertex, it might as well be a leaf only
			{
				resultList.addEdgeReplacer(true, remainingNode, afterLastRemoveNode, path.getPathVertex(removeEnd), afterLastRemoveNode);
				removeSubPath(g, path, removeStart, removeEnd);
			}


		}
		return 0;
	}

	@Override
	public String getName() {
		return "Path-Reduction-Rule 3: Contract path";
	}
}
