package reductionRules.path;

import result.ResultList;
import main.Degree2Path;
import main.Graph;
import main.Node;

/**
 * An abstract representation for a Path Reduction Rule that removes Y-graphs.
 * Currently these are Path Reduction Rule 1 and Path Reduction Rule 2.
 * In a path that is not a simples circle the rules will be applied in both directions, to make sure all Y-graphs are removed
 * @author Steffen Toborg
 */
public abstract class PRR_RemoveYGraph extends PathReductionRule 
{
	@Override
	protected int applyRule(Graph g, Degree2Path path, ResultList resultList) 
	{
		boolean isCircle = path.isSimpleCircle();
		if(isCircle && path.pathNodes.size()==3 && deletesConnectingPoint())
			return 0;//no deletion possible because circle too small
		
		int kReduced = 0;
		//loop over the inner path nodes
		//until the second last inner-path-node, because the next node must be an inner-path-node
		//if the path is a circle then start with the first node because it is an inner path node as well
		for(int i = isCircle?0:1 ; i < (isCircle?path.pathNodes.size():path.pathNodes.size()-2); i++)
		{
			Node ni = path.pathNodes.get(i);
			
			//if(ni.Y.size() == 1)
			if(PathRedLPYSet.hasYGraph(ni))
			{
				Node ni_1 = path.getPathVertex(i-1);
				Node ni1 = path.getInnerPathVertex(i+1);//v_i+1
				Node ni2 = path.getInnerPathVertex(i+2);
				if(applyRule(g, ni_1, ni, ni1, ni2, resultList))
				{
					if(deletesConnectingPoint())
					{
						path.removeNodesFromPath(i,i);
						i--;
					}
					kReduced++;
				}
			}
		}

		if(!isCircle)//in a circle only one direction is nescessary
		{
			for(int i = path.pathNodes.size()-2; i > 1; i--)
			{
				Node ni = path.pathNodes.get(i);
				//if(ni.Y.size() == 1)
				if(PathRedLPYSet.hasYGraph(ni))
				{
					Node ni_1 = path.getPathVertex(i+1);
					Node ni1 = path.getInnerPathVertex(i-1);//v_i+1
					Node ni2 = path.getInnerPathVertex(i-2);
					if(applyRule(g, ni_1, ni, ni1, ni2, resultList))
					{
						//tokens don't have to be reset here because the should not have been computed at this point anyway
						if(deletesConnectingPoint())
							path.removeNodesFromPath(i,i);
						
						kReduced++;
					}
				}
			}
		}
		return kReduced;
	}
	abstract protected boolean applyRule(Graph g, Node ni_1, Node ni, Node ni1, Node ni2, ResultList resultList);

	/**
	 * denotes whether this rule should delete the connection point ni on success 
	 * @return
	 */
	abstract protected boolean deletesConnectingPoint();
}
