package reductionRules.path;

import java.util.HashSet;

import main.Degree2Path;
import main.Graph;
import main.Node;
import main.SolvingState;
import reductionRules.ReductionRule;
import result.ResultList;
import Utils.ExpandMode;
import Utils.GraphUtils;

/**
 * An abstract representation for a Path Reduction Rule
 * @author Steffen Toborg
 */
public abstract class PathReductionRule extends ReductionRule{

	public int apply(Graph g, Degree2Path path, ResultList resultList)
	{
		//ArrayList<Node> pNodes_old = new ArrayList<Node>(path.pathNodes);
		
		OnBeforeChange(g, path);
		int returnVal = applyRule(g, path, resultList);
		onAfterChange(g, path);
		return returnVal;
	}
	
	protected HashSet<Node> removeSubPath(Graph g, Degree2Path path, int start, int end)
	{
		HashSet<Node> removed = path.getPendantTreeNodes(start, end);
		onEdgeAdded(path.getPathVertex(start-1), path.getPathVertex(end+1));//this edge is added within the following method 
		path.removeNodesFromPath(start, end);

		
		deleteNodes(g, removed, false);
		return removed;
	}
	
	
	/**
	 * Deletes one Y-Graph which is a subgraph of the connectingNode
	 * The Connecting Node itself will not be deleted
	 * @throws NullPointerException if there is no Y-Graph as a subGraph of the connectingNode
	 * @param g
	 * @param connectingNode
	 */
	protected Node deleteYGraph(Graph g, Node connectingNode, boolean deleteConnectingEdge)
	{
		Node ny = PathRedLPYSet.getYGraph(connectingNode);

		HashSet<Node> toDelete = GraphUtils.expand(ny, connectingNode, ExpandMode.nonCyclic);
		
		deleteNodes(g, toDelete, false);

		if(deleteConnectingEdge)
			GraphUtils.removeEdge(connectingNode, ny);
		return ny;
	}
	
	/**
	 * Applies the data-reduction-rule to the given Graph and Degree2Path
	 * @param g
	 * @param path
	 * @return
	 */
	protected abstract int applyRule(Graph g, Degree2Path path, ResultList resultList);
	
	@Override
	public SolvingState getNormalState()
	{
		return SolvingState.PRR;
	}
}
