package reductionRules.tree;

import java.util.HashSet;

import result.ResultList;

import Utils.ExpandMode;
import Utils.GraphUtils;
import main.Graph;
import main.Node;

/**
 * Tree Reduction Rule 2:
 * Let n be a vertex with |Y(n)| >= 1 and |Y(n)| + |L(n)| + |P2(n)| >= 2.
 * Then, for an arbitrary chosen Y-graph Y with connection-point n, delete all vertices of Y except for n and decrease k by one.
 * @author Steffen Toborg
 */
public class TRR2_CutOffYGraph extends TreeReductionRule 
{
	@Override
	protected int applyRule(Graph g, Node n, TreeRedLPYSet thisLPY, ResultList resultList)
	{
		int reducedK = 0;
		while(thisLPY.Y.size() >= 1 && thisLPY.Y.size()+thisLPY.P2.size()+thisLPY.L.size() >= 2)
		{
			Node y = thisLPY.Y.poll();
			
			GraphUtils.removeEdge(n, y);
			
			HashSet<Node> yGraphNodes = GraphUtils.expand(y, n, ExpandMode.nonCyclic);
			deleteNodes(g, yGraphNodes, false);
			
			resultList.add(true, n, y);
			reducedK++;
		}
		return reducedK;
	}
	
	@Override
	public String getName() {
		return "Tree-Reduction-Rule 2: Cut off Y-graph";
	}
}
