package reductionRules.tree;

import result.ResultList;
import main.Graph;
import main.Node;
import Utils.ExpandMode;
import Utils.GraphUtils;

/**
 * Tree Reduction Rule 3:
 * Let n be a vertex with |P2(n)| >= 3,
 * then delete every P2 p in P2(n) but two and decrease k by the number of deleted P2s
 * @author Steffen Toborg
 */
public class TRR3_Reduce2Claws extends TreeReductionRule 
{
	/**
	 * Tree Reduction Rule 3:
	 * Let n be a vertex with |P2(n)| >= 3,
	 * then delete every P2 p in P2(n) but two and decrease k by the number of deleted P2s
	 */
	@Override
	protected int applyRule(Graph g, Node n, TreeRedLPYSet thisLPY, ResultList resultList)
	{
		int p2count = thisLPY.P2.size();
		while(thisLPY.P2.size() >= 3)
		{
			Node np2 = thisLPY.P2.poll();
			deleteNode(g, np2, false);//delete P2
			GraphUtils.removeEdge(n, np2);
			deleteNode(g, GraphUtils.getOtherNeighbour(np2, n, ExpandMode.nonCyclic), false);//delete leaf of P2 (there is exactly one)
			
			resultList.add(true, n, np2);
		}
		return Math.max(0, p2count-2);
	}

	@Override
	public String getName() {
		return "Tree-Reduction-Rule 3: Reduce 2-claws";
	}
}
