package reductionRules.tree;


import result.ResultList;
import main.Graph;
import main.Node;
import Utils.ExpandMode;
import Utils.GraphUtils;

/**
 * Tree Reduction Rule 4:
 * Let n be a vertex with ^deg_G(n)=2(non-leaf-degree) and let p be a P2 with P2(n)={p}. Then delete the Vertex l(p).
 * @author Steffen Toborg
 */
public class TRR4_RemoveEndPoint extends TreeReductionRule 
{
	@Override
	protected int applyRule(Graph g, Node n, TreeRedLPYSet thisLPY, ResultList resultList) {

		//if non-leaf-degree != 2 continue
		if(GraphUtils.getNonLeafDegree(n) == 2 && thisLPY.P2.size()==1)
		{
			Node np2 = thisLPY.P2.poll();
			deleteNode(g, GraphUtils.getOtherNeighbour(np2, n, ExpandMode.nonCyclic), true);//delete leaves of P2

			//only add to leaves list is there is no leaf yet
			//if there is a leaf then this newly added leaf should be deleted any by TRR1
			if(thisLPY.L.size()==0)
				thisLPY.L.add(np2);
			else
				deleteNode(g, np2, true);
		}
		return 0;
	}

	@Override
	public String getName() {
		return "Tree-Reduction-Rule 4: Remove end point";
	}
}
