package reductionRules.tree;

import java.util.HashSet;

import result.ResultList;

import main.Graph;
import main.Node;
import Utils.ExpandMode;
import Utils.GraphUtils;

/**
 * Tree Reduction Rule 5:
 * Let n be a vertex with deg_G(n)=2 and let Y be a Y-graph such that Y(n)={Y}.
 * Then, delete all vertexes of Y (including n) and decrease k by one.
 * @author Steffen Toborg
 */
public class TRR5_RemoveEndYGraph extends TreeReductionRule {

	@Override
	protected int applyRule(Graph g, Node n, TreeRedLPYSet thisLPY, ResultList resultList) 
	{
		if(thisLPY.Y.size() == 1 && n.getNeighbourCount() == 2 && !n.isCyclic)
		{
			Node y = thisLPY.Y.poll();


			HashSet<Node> yGraphNodes = GraphUtils.expand(y, n, ExpandMode.nonCyclic);
			deleteNodes(g, yGraphNodes, false);

			Node otherNeighbour = GraphUtils.getOtherNeighbour(n, y, ExpandMode.both);

			//if the parent of this node is a leaf then instead delete the edge between y and n
			if(otherNeighbour.isLeaf())
			{
				GraphUtils.removeEdge(n, y);
				resultList.add(true, n, y);
			}
			else
			{
				resultList.add(true, n, otherNeighbour);
				//n cannot be removed from it's parent's neighbourlist yet because this would result in an concurrentModificationException
				//but it will be set to deleted = true and thus be removed after the rule is applied
				deleteNode(g, n, false);
			}
			return 1;
		}
		else
			return 0;
	}

	@Override
	public String getName() {
		return "Tree-Reduction-Rule 5: Remove end Y-graph";
	}
}
