package reductionRules.tree;


import main.Graph;
import main.Node;
import main.SolvingState;
import reductionRules.ReductionRule;
import result.ResultList;

/**
 * An abstract representation for a Tree Reduction Rule
 * @author Steffen Toborg
 */
public abstract class TreeReductionRule extends ReductionRule {
	
	/**
	 * Applies this data-reduction-rule to the given Graph and Node.
	 * And invokes onBeforeChanges before and onAfterChange after applying the rule.
	 * The Parameters L, P2, Y should be set before this call for all neighbours that are below n in the relevant sub-tree
	 * @param g the Graph
	 * @param g the Node
	 * @param thisLPY 
	 * @return the value by which k has to be reduced	
	 */
	public int apply(Graph g, Node n, TreeRedLPYSet thisLPY, ResultList resultList)
	{
		OnBeforeChange(g, n);
		int returnVal = applyRule(g, n, thisLPY, resultList);
		onAfterChange(g, n);
		return returnVal;
	}
	
	/**
	 * Applies this data-reduction-rule to the given Graph and Node.
	 * This Method must be overridden by every sub-class, specifying the concrete reduction.
	 * The Parameters L, P2, Y should be set before this call for all neighbours that are below n in the relevant sub-tree
	 * @param g the Graph
	 * @param g the Node
	 * @return the value by which k has to be reduced	
	 */
	protected abstract int applyRule(Graph g, Node n, TreeRedLPYSet thisLPY, ResultList resultList);
	
	@Override
	public SolvingState getNormalState()
	{
		return SolvingState.TRR;
	}
}
