package result;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;

import main.Node;
import main.Start;

import Utils.Tuple;

public class ReSultList_alt implements Iterable<String>{
	ArrayList<Tuple<String, String>> deletedEdges;

	ArrayList<ConditionedEdge> level1CondEdges;
	ArrayList<ConditionedEdge> level2CondEdges;

	int size;

	public ReSultList_alt() {
		deletedEdges = new ArrayList<Tuple<String,String>>();
		level1CondEdges = new ArrayList<ReSultList_alt.ConditionedEdge>();
		level2CondEdges = new ArrayList<ReSultList_alt.ConditionedEdge>();
		size = 0;
	}
	public void add(Node n1, Node n2)
	{
		if(Start.debugMode)
		{
			if(deletedEdges.contains(getEdge(n1,n2)))
					throw new IllegalArgumentException();
		}
		size++;
		deletedEdges.add(getEdge(n1,n2));
	}

	public void addAll(ReSultList_alt toAdd) {
		size += toAdd.size;
		deletedEdges.addAll(toAdd.deletedEdges);
		level1CondEdges.addAll(toAdd.level1CondEdges);
		level2CondEdges.addAll(toAdd.level2CondEdges);
	}

	public void addEdgeReplacer(boolean level1, Node c1, Node c2, Node realEdgeNode1, Node realEdgeNode2)
	{
		ArrayList<Tuple<Node, Node>> ifEdgeDel = new ArrayList<Tuple<Node,Node>>();
		ifEdgeDel.add(new Tuple<Node, Node>(realEdgeNode1, realEdgeNode2));
		addEdgeReplacer(level1, c1, c2, ifEdgeDel, new ArrayList<Tuple<Node, Node>>());
	}
	public void addEdgeReplacer(boolean level1, Node c1, Node c2, Collection<Tuple<Node, Node>> ifEdgeDeleted, Node ifNotDel1, Node ifNotDel2)
	{
		ArrayList<Tuple<Node, Node>> ifNotDel = new ArrayList<Tuple<Node,Node>>();
		ifNotDel.add(new Tuple<Node, Node>(ifNotDel1, ifNotDel2));
		addEdgeReplacer(level1, c1, c2, ifEdgeDeleted, ifNotDel);//count edges in next step
	}
	public void addEdgeReplacer(boolean level1, Node c1, Node c2, Collection<Tuple<Node, Node>> ifEdgeDeleted, Collection<Tuple<Node, Node>> ifNotDeleted)
	{
		size += ifNotDeleted.size();
		if(level1)
			level1CondEdges.add(new ConditionedEdge(c1, c2, ifEdgeDeleted, ifNotDeleted));
		else
			level2CondEdges.add(new ConditionedEdge(c1, c2, ifEdgeDeleted, ifNotDeleted));

	}

	public int size() {
		return size;
	}

	@Override
	public Iterator<String> iterator() {
		return new EdgeStringIterator();
	}

	private class ConditionedEdge 
	{
		//String c1;
		//String c2;
		Tuple<String,String> conditionEdge;
		ArrayList<Tuple<String, String>> ifEdgeDeleted;
		ArrayList<Tuple<String, String>> ifNotDeleted;
		public ConditionedEdge(Node c1, Node c2, Collection<Tuple<Node, Node>> ifEdgeDeleted, Collection<Tuple<Node, Node>> ifNotDeleted)
		{
			this.conditionEdge = getEdge(c1, c2);

			this.ifEdgeDeleted = new ArrayList<Tuple<String,String>>(ifEdgeDeleted.size());
			for (Tuple<Node, Node> tuple : ifEdgeDeleted)
				this.ifEdgeDeleted.add(getEdge(tuple.left, tuple.right));

			this.ifNotDeleted = new ArrayList<Tuple<String,String>>(ifNotDeleted.size());
			for (Tuple<Node, Node> tuple : ifNotDeleted)
				this.ifNotDeleted.add(getEdge(tuple.left, tuple.right));
		}
	}
	private Tuple<String, String> getEdge(Node n1, Node n2)
	{
		return new Tuple<String, String>(n1.name, n2.name);
	}

	public HashSet<Tuple<String,String>> getFinalEdgesSet()
	{
		HashSet<Tuple<String, String>> returnVal = new HashSet<Tuple<String,String>>(deletedEdges);

		for(ConditionedEdge cEdge : level1CondEdges) 
		{
			if(returnVal.contains(cEdge.conditionEdge))
			{
				returnVal.remove(cEdge.conditionEdge);
				returnVal.addAll(cEdge.ifEdgeDeleted);
			}
			else
				returnVal.addAll(cEdge.ifNotDeleted);
		}

		for(ConditionedEdge cEdge : level2CondEdges) 
		{
			if(returnVal.contains(cEdge.conditionEdge))
			{
				returnVal.remove(cEdge.conditionEdge);
				returnVal.addAll(cEdge.ifEdgeDeleted);
			}
			else
				returnVal.addAll(cEdge.ifNotDeleted);
		}
		return returnVal;
	}

	private class EdgeStringIterator implements Iterator<String>
	{
		Iterator<Tuple<String, String>> edgesIterator;
		public EdgeStringIterator() {
			edgesIterator = getFinalEdgesSet().iterator();
		}
		@Override
		public boolean hasNext() {
			return edgesIterator.hasNext();
		}

		@Override
		public String next() {
			Tuple<String,String> edge = edgesIterator.next();
			return edge.left+" "+edge.right;
		}

		@Override
		public void remove() {
			throw new UnsupportedOperationException();
		}

	}
}
