package result;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;

import main.Node;
import Utils.Tuple;

public class ResultList implements Iterable<String>{
	//ArrayList<Tuple<String, String>> deletedEdges;

	//ArrayList<ConditionedEdge> level1CondEdges;
	//ArrayList<ConditionedEdge> level2CondEdges;


	ArrayList<ResultListEntry> entries;

	int size;

	public int getReducedCount()
	{
		int reduced = 0;
		for(ResultListEntry entry : entries)
		{
			if(entry.addedByDataReduction)
				reduced += entry.getSize();
		}
		return reduced;
	}

	public ResultList() {
		entries = new ArrayList<ResultListEntry>();
		size = 0;
	}
	public void add(boolean addedByDataReduction, Node n1, Node n2)
	{

		entries.add(new DeletedEdgeEntry(addedByDataReduction, n1.name, n2.name));
		size++;
		//		deletedEdges.add(getEdge(n1,n2));
	}

	public void add(ResultListEntry entry)
	{
		entries.add(entry);
		size++;
	}

	public void addAll(ResultList toAdd) {
		size += toAdd.size;
		entries.addAll(toAdd.entries);
		//		deletedEdges.addAll(toAdd.deletedEdges);
		//		level1CondEdges.addAll(toAdd.level1CondEdges);
		//		level2CondEdges.addAll(toAdd.level2CondEdges);
	}

	public void addEdgeReplacer(boolean addedByDataReduction, Node c1, Node c2, Node realEdgeNode1, Node realEdgeNode2)
	{
		ArrayList<Tuple<Node, Node>> ifEdgeDel = new ArrayList<Tuple<Node,Node>>();
		ifEdgeDel.add(new Tuple<Node, Node>(realEdgeNode1, realEdgeNode2));
		addEdgeReplacer(addedByDataReduction, c1, c2, ifEdgeDel, new ArrayList<Tuple<Node, Node>>());
	}
	public void addEdgeReplacer(boolean addedByDataReduction, Node c1, Node c2, Collection<Tuple<Node, Node>> ifEdgeDeleted, Node ifNotDel1, Node ifNotDel2)
	{
		ArrayList<Tuple<Node, Node>> ifNotDel = new ArrayList<Tuple<Node,Node>>();
		ifNotDel.add(new Tuple<Node, Node>(ifNotDel1, ifNotDel2));
		addEdgeReplacer(addedByDataReduction, c1, c2, ifEdgeDeleted, ifNotDel);//count edges in next step
	}
	public void addEdgeReplacer(boolean addedByDataReduction, Node c1, Node c2, Collection<Tuple<Node, Node>> ifEdgeDeleted, Collection<Tuple<Node, Node>> ifNotDeleted)
	{
		size += ifNotDeleted.size();
		entries.add(new EdgeReplacer(addedByDataReduction, c1, c2, ifEdgeDeleted, ifNotDeleted));
	}

	public void addEdgeReplacer(boolean addedByDataReduction, Node c1, Node c2, Node d1, Node d2, Collection<Tuple<Node, Node>> ifEdgeDeleted, Collection<Tuple<Node, Node>> ifNotDeleted)
	{
		size += ifNotDeleted.size();
		entries.add(new EdgeReplacer(addedByDataReduction, c1, c2, d1, d2, ifEdgeDeleted, ifNotDeleted));
	}

	public int size() {
		return size;
	}

	@Override
	public Iterator<String> iterator() {
		return new EdgeStringIterator();
	}

	public HashSet<Tuple<String,String>> getFinalEdgesSet()
	{
		HashSet<Tuple<String, String>> returnVal = new HashSet<Tuple<String,String>>();

		Collections.sort(entries);

		//		for (int i = 0; i < entries.size(); i++) 
		for (int i = entries.size()-1; i >= 0; i--) 
		{
			ResultListEntry entry = entries.get(i);

			entry.addDeletedEdges(returnVal);
		}

		return returnVal;
	}

	private class EdgeReplacer extends ResultListEntry
	{
		//String c1;
		//String c2;
		Tuple<String,String> conditionEdge;
		Tuple<String,String> conditionEdge2;
		ArrayList<Tuple<String, String>> ifEdgeDeleted;
		ArrayList<Tuple<String, String>> ifNotDeleted;
		public EdgeReplacer(boolean addedByDataReduction, Node c1, Node c2, Collection<Tuple<Node, Node>> ifEdgeDeleted, Collection<Tuple<Node, Node>> ifNotDeleted)
		{
			super(addedByDataReduction);
			this.conditionEdge = getEdge(c1, c2);

			this.ifEdgeDeleted = new ArrayList<Tuple<String,String>>(ifEdgeDeleted.size());
			for (Tuple<Node, Node> tuple : ifEdgeDeleted)
				this.ifEdgeDeleted.add(getEdge(tuple.left, tuple.right));

			this.ifNotDeleted = new ArrayList<Tuple<String,String>>(ifNotDeleted.size());
			for (Tuple<Node, Node> tuple : ifNotDeleted)
				this.ifNotDeleted.add(getEdge(tuple.left, tuple.right));
		}

		public EdgeReplacer(boolean addedByDataReduction, Node c1, Node c2, Node d1, Node d2,  Collection<Tuple<Node, Node>> ifEdgeDeleted, Collection<Tuple<Node, Node>> ifNotDeleted)
		{
			super(addedByDataReduction);
			this.conditionEdge = getEdge(c1, c2);

			this.conditionEdge2 = getEdge(d1, d2);


			this.ifEdgeDeleted = new ArrayList<Tuple<String,String>>(ifEdgeDeleted.size());
			for (Tuple<Node, Node> tuple : ifEdgeDeleted)
				this.ifEdgeDeleted.add(getEdge(tuple.left, tuple.right));

			this.ifNotDeleted = new ArrayList<Tuple<String,String>>(ifNotDeleted.size());
			for (Tuple<Node, Node> tuple : ifNotDeleted)
				this.ifNotDeleted.add(getEdge(tuple.left, tuple.right));
		}

		@Override
		public void addDeletedEdges(HashSet<Tuple<String, String>> currentSet) {
			if(currentSet.contains(conditionEdge))
			{
				currentSet.remove(conditionEdge);
				currentSet.addAll(ifEdgeDeleted);
			}
			else if(currentSet.contains(conditionEdge2))
			{
				currentSet.remove(conditionEdge2);
				currentSet.addAll(ifEdgeDeleted);
			}
			else
				currentSet.addAll(ifNotDeleted);
		}
		private Tuple<String, String> getEdge(Node n1, Node n2)
		{
			return new Tuple<String, String>(n1.name, n2.name);
		}

		@Override
		public int getSize() {
			return ifNotDeleted.size();
		}
	}

	private class EdgeStringIterator implements Iterator<String>
	{
		Iterator<Tuple<String, String>> edgesIterator;
		public EdgeStringIterator() {
			edgesIterator = getFinalEdgesSet().iterator();
		}
		@Override
		public boolean hasNext() {
			return edgesIterator.hasNext();
		}

		@Override
		public String next() {
			Tuple<String,String> edge = edgesIterator.next();
			return edge.left+" "+edge.right;
		}

		@Override
		public void remove() {
			throw new UnsupportedOperationException();
		}

	}
}
