package solving;

import java.awt.IllegalComponentStateException;

import main.Graph;
import main.Node;
import main.SolvingState;
import reductionRules.Reduction;
import result.ResultList;
import solving.newSolver.BSS1_ThreeCycles;
import solving.newSolver.BSS2_Tokens;
import solving.newSolver.BSS36_Claws;
import solving.newSolver.NewBranchFinisher;
import visualization.GraphChanger;
import visualization.GraphComponent;

public abstract class BranchSolverStep extends GraphChanger
{
	public static int recsteps;
	public static int startBranching(Graph g, ResultList resultList)
	{
//		int maxK = 0; //TODO: revert to k = 0
//		int initialSize = resultList.size();
		g.findAndSetThreeCycles();

		//TODO: algorithm does not depend on k, rewrite to compute minimum k
		//directly, avoiding exhaustive search
		return step1ThreeCycles.start(g, 0, Integer.MAX_VALUE, resultList);
		//		while(true)
		//		{
		//			if(step1ThreeCycles.start(g, 0, maxK, resultList) != -1)
		//				return maxK;
		//			else
		//				if(resultList.size() != initialSize)
		//					throw new IllegalComponentStateException();
		//			maxK++;
		//		}
	}


	protected static BranchSolverStep step1ThreeCycles = new BSS1_ThreeCycles();
	protected static BranchSolverStep step2Tokens = new BSS2_Tokens();
	protected static BranchSolverStep step3Claws = new BSS36_Claws();
	protected static BranchSolverStep finisher = new NewBranchFinisher();


	/**
	 * Starts Branching with this step. If method returns false the Graph should not be modified in any way.
	 * @param g
	 * @param k
	 * @param resultList
	 * @return true is branching was successfull, else false
	 */
	public abstract int start(Graph g, int k, int maxK, ResultList resultList);

	public BranchSolverStep() 
	{
	}


	protected abstract int goToNextStep(Graph g, BranchSolverStep defaultStep, int k, int maxK, ResultList resultList);
	//	{
	//		ResultList reductionResult = new ResultList();
	//		int reduced = onLeaveStep(g, reductionResult);
	//		return getNextStep(g).start(g, k, resultList);
	//	}

	/*protected boolean goToNextStep(BranchSolverStep defaultStep, Graph g, int k, ResultList resultList)
	{
		if(g.hasThreeCycles() && this != step1ThreeCycles)
		{
			//onLeaveStep(g);
			return step1ThreeCycles.start(g, k, resultList);
		}
		else
		{
			GraphCopy copy = new GraphCopy(g);
			ResultList reductionResult = new ResultList();

			int reduced = onLeaveStep(copy, reductionResult);
			if(defaultStep.start(copy, k-reduced, reductionResult))
			{
				resultList.addAll(reductionResult);
				return true;
			}
			else
				return false;
		}
	}*/

	protected class Edge implements GraphComponent{
		Node n1, n2;
		public Edge(Node n1, Node n2) {
			this.n1 = n1;
			this.n2 = n2;
		}
		@Override
		public boolean contains(Node n) {
			return n1==n || n2==n;
		}
	}

	public SolvingState getNormalState()
	{
		return SolvingState.BRANCHING;
	}

	/*protected int onLeaveStep(Graph g, ResultList resultList)
	{
		return 0;
	}*/

	//	@Override
	//	protected void OnBeforeChange(Graph g, GraphComponent d)
	//	{
	//		super.OnBeforeChange(g, d);
	//		if(Start.useBranchTreeVisualization){
	//			BranchTreeViewing.onNewBranch(d, this);
	//		}
	//	}

	//	protected void onBranchReturn()
	//	{
	//		if(Start.useBranchTreeVisualization){
	//			BranchTreeViewing.onBranchReturn();
	//		}
	//	}
}
