package solving;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Stack;

import main.Graph;
import main.Node;
import Utils.ConnCompIterator;

public class GraphBBridgesInfo {
	boolean[][] edgeIsInCircle;
	boolean[] nodeIsInCircle;
	public GraphBBridgesInfo(Graph g) 
	{
		edgeIsInCircle = new boolean[g.initialNodesCount][g.initialNodesCount];
		nodeIsInCircle = new boolean[g.initialNodesCount];
		
		ConnCompIterator connCompIt = new ConnCompIterator(g);
		while(connCompIt.hasNext())
		{
			HashSet<Node> nodeSet = connCompIt.next();
			Node cyclic = null;
			for(Node n : nodeSet)
			{
				if(n.isCyclic)
				{
					cyclic = n;
					break;
				}
			}
			if(cyclic != null)
			{
				markBridgeEdges_branch(cyclic, null, new HashMap<Node, Integer>(), new Stack<Node>());
			}
		}
	}

	private Node markBridgeEdges_branch(Node n, Node prevNode,HashMap<Node, Integer> nodeToIndex, Stack<Node> currentPath)
	{
		//if(currentPath.contains(n))
		if(nodeToIndex.containsKey(n))
		{
			return n;
		}

		currentPath.push(n);
		nodeToIndex.put(n, currentPath.size());

		Node returnVal = null;
		int returnValIndex = -1;
		for(Node neighbour : n.cyclicNeighbours)
		{
			//prevent goint backwards
			if(neighbour == prevNode || isInCycle(n, neighbour))
				continue;

			Node cyclicNode = markBridgeEdges_branch(neighbour, n, nodeToIndex, currentPath);

			if(cyclicNode != null)
			{
				setIsInCycle(n, neighbour);
				int cyclicNodeIndex = nodeToIndex.get(cyclicNode);
				if(returnVal == null ||cyclicNodeIndex < returnValIndex)
				{
					returnVal = cyclicNode;
					returnValIndex = cyclicNodeIndex;
				}
			}
		}
		if(returnVal == n)
			returnVal = null;

		currentPath.pop();
		nodeToIndex.remove(n);
		
		return returnVal;
	}
	
	private void setIsInCycle(Node n1, Node n2)
	{
		nodeIsInCircle[n1.index] = nodeIsInCircle[n2.index] = true;
		edgeIsInCircle[n1.index][n2.index] = edgeIsInCircle[n2.index][n1.index] = true;
	}
	
	private boolean isInCycle(Node n1, Node n2)
	{
		return edgeIsInCircle[n1.index][n2.index];
	}
	

	public boolean isAdjacentToBBridge(Node n)
	{
		for (Node neighbour : n.cyclicNeighbours) {
			if(isBBridgeEdge(n,neighbour))
				return true;
		}
		return false;
	}
	public boolean isBBridgeNode(Node n)
	{
		return !nodeIsInCircle[n.index];
	}
	public boolean isBBridgeEdge(Node n1, Node n2)
	{
		return !edgeIsInCircle[n1.index][n2.index];
	}
}
