package solving.circlesTree;


import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;

import main.Node;
import Utils.ExpandMode;
import Utils.GraphUtils;

public class ComponentTreeNode{

	public HashSet<Node> nodes;

	public HashSet<Node> anchorNodes;
	Hashtable<Node, Node> anchorToFirstOuter;
	boolean isValidCircle;

	public ComponentTreeNode(HashSet<Node> nodes) {
		this.nodes = nodes;
		isValidCircle = true;

		anchorNodes = new HashSet<Node>();
		anchorToFirstOuter = new Hashtable<Node, Node>();
		for(Node n : nodes)
		{
			Node outerNeighbour = GraphUtils.getOtherNeighbour(n, nodes, ExpandMode.cyclic);
			
			if(outerNeighbour != null)
			{
				anchorNodes.add(n);
				anchorToFirstOuter.put(n, outerNeighbour);
			}
		}
	}

	protected void removeDeletedNodes()
	{
		for (Iterator<Node> iterator = nodes.iterator(); iterator.hasNext();) {
			Node n = iterator.next();
			if(n.deleted)
				iterator.remove();
		}
	}

	public Node getFirstOuterCircleNodeForAnchor(Node anchor)
	{
		return anchorToFirstOuter.get(anchor);
	}

	public boolean isCircle() {
		for(Node n : nodes)
		{
			if(n.cyclicNeighbours.size() > 2 && !anchorNodes.contains(n))
				return false;
		}
		return true;
	}
}
