package solving.circlesTree;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;

import main.Graph;
import main.Node;
import solving.GraphBBridgesInfo;
import Utils.ExpandMode;
import Utils.ExpansionRestrictor;
import Utils.GraphUtils;
import Utils.MarkingIterator;

public class ComponentsTree {
	private Graph g;
	public ArrayList<ComponentTreeNode> components;
	//private Hashtable<Node, ComponentTreeNode> anchorToCircleNode;

	public ComponentsTree(Graph g, Collection<Node> nodes) 
	{
		this.g = g;
		this.components = new ArrayList<ComponentTreeNode>();
		//anchorToCircleNode = new Hashtable<Node, ComponentTreeNode>();

		GraphBBridgesInfo bridgeInfo = new GraphBBridgesInfo(g);

		MarkingIterator<Node> markingIt = new MarkingIterator<Node>(nodes);
		while(markingIt.hasNext())
		{
			Node n = markingIt.next();
			if(n.isCyclic)
			{
				HashSet<Node> component = expandToComponent(n, bridgeInfo);

				if(component.size() > 1)
				{
					markingIt.markAll(component);
					components.add(new ComponentTreeNode(component));
				}
			}
		}
		/*for(Node n : g.nodes)
		{
			if(n.isCyclic)
			{
				cyclic = n;
				break;
			}
		}
		if(cyclic != null)
		{
			markBridgeEdges_branch(nodes.iterator().next(), null, new HashSet<Node>(), new LinkedList<Node>());
		}*/
	}

	private HashSet<Node> expandToComponent(Node n, final GraphBBridgesInfo bBridgesInfo)
	{
		ExpansionRestrictor noBBridges = new ExpansionRestrictor() {
			@Override
			public boolean allowExpand(Node current, Node neighbour) {
				return !bBridgesInfo.isBBridgeEdge(current, neighbour);
			}
		};

		HashSet<Node> component = GraphUtils.expand(n, noBBridges, ExpandMode.cyclic);
		return component;
	}

	/*private CirclesTreeNode markBridgeEdges_branch(Node n, Node prevNode, HashSet<Node> visited, LinkedList<Node> currentPath)
	{
		if(currentPath.contains(n))
		{
			CirclesTreeNode newNode = new CirclesTreeNode();
			newNode.nodes.add(n);
			return newNode;
		}

		currentPath.addLast(n);

		CirclesTreeNode returnVal = null;
		for(Node neighbour : n.cyclicNeighbours)
		{
			//prevent goint backwards
			if(neighbour == prevNode)
				continue;

			if(visited.contains(neighbour))
				continue;

			CirclesTreeNode result = markBridgeEdges_branch(neighbour, n, visited, currentPath);
			if(result != null)
			{
				if(returnVal != null)
					returnVal.isValidCircle = false;
				else
					returnVal = result;
			}
		}
		if(returnVal != null)
		{
			if(returnVal.nodes.get(0) == n)
			{
				returnVal.onCompleted();
				for(Node anchor : returnVal.anchorNodes)
					anchorToCircleNode.put(anchor, returnVal);
				circles.add(returnVal);
				returnVal = null;
			}
			else
				returnVal.nodes.add(n);
		}

		currentPath.removeLastOccurrence(n);
		return returnVal;
	}*/

	public ArrayList<ComponentTreeNode> findLeaves() {
		ArrayList<ComponentTreeNode> result = new ArrayList<ComponentTreeNode>();
		for(ComponentTreeNode node : components)
		{
			if(node.anchorNodes.size() == 1)
			{
				Node anchor = node.anchorNodes.iterator().next();
				if(anchor.cyclicNeighbours.size() == 3)//TODO schoener machen
					result.add(node);
			}
		}
		return result;
	}
}
