package solving.circlesTree;

import java.awt.IllegalComponentStateException;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import main.Degree2Path;
import main.Node;
import main.Path;
import main.TwoNodesPath;
import visualization.GraphComponent;
import Utils.GraphUtils;
import Utils.Tuple;

public class PathBasedCirclesTree 
{
	public LinkedList<PathBasedCircleTreeNode> nodes;

	public PathBasedCirclesTree(Collection<Node> originalNodes, List<Degree2Path> arrayList) 
	{
		nodes = new LinkedList<PathBasedCircleTreeNode>();

		if(arrayList.size() == 1)
		{
			Degree2Path p = arrayList.get(0);	
			if(!p.isSimpleCircle())
				throw new IllegalComponentStateException("Connected component contains exycly one degree-2-path, and it is not a circle");
			PathBasedCircle circle = new PathBasedCircle(p.getStart(), new LinkedList<Path>(arrayList));
			circle.setComplete();
			nodes.add(circle);
			return;
		}

		Node start = null;
		for(Node n : originalNodes)
		{
			if(n.isCyclic && n.cyclicNeighbours.size() >= 3)
				start = n;
		}

		if(start == null)
			return;

		findCircles_branch(start, null, new LinkedList<GraphComponent>());
	}

	private PathBasedCircleTreeNode findCircles_branch(Node n, Path prev, LinkedList<GraphComponent> currentPath)
	{
		//assume that all circles are vertex disjoint
		if(currentPath.contains(n))
		{
			LinkedList<Path> c = new LinkedList<Path>();
			return new PathBasedCircle(n,c);
		}

		currentPath.addLast(n);

		LinkedList<Tuple<Path, PathBasedCircleTreeNode>> edges = new LinkedList<Tuple<Path,PathBasedCircleTreeNode>>();

		PathBasedCircleTreeNode circle = null;

		for(Node neighbour : n.cyclicNeighbours)
		{
			//prevent goint backwards
			if(prev != null && prev.isStartOrEnd(neighbour))
				continue;

			//these paths are visited via the degree2Path that must exist here
			if(neighbour.cyclicNeighbours.size() < 3)
				continue;

			//TwoNodesPath neighbourPath = new TwoNodesPath(n, neighbour);

			PathBasedCircleTreeNode c = branchOnPath(n, new TwoNodesPath(n, neighbour), neighbour, edges, currentPath);
			if(c != null)
				circle = c;
		}
//		for(Degree2Path neighbourPath : n.degree2Paths)//TODO
//		{
//			if(neighbourPath == prev)
//				continue;
//
//			PathBasedCircleTreeNode c = branchOnPath(n, neighbourPath, GraphUtils.getOtherEndNode(neighbourPath, n), edges, currentPath);
//			if(c != null)
//				circle = c;
//		}

		if(circle == null)
		{
			circle = new SimpleNode(n);
			nodes.add(circle);
		}
		else
		{
			if(circle.getFirstNode() == n)
			{
				circle.setComplete();
				nodes.add(circle);
			}
			//else
			//circle.addToPath(prev);
		}
		circle.neighbours.addAll(edges);
		for(Tuple<Path, PathBasedCircleTreeNode> e : edges)
			e.right.neighbours.add(new Tuple<Path, PathBasedCircleTreeNode>(e.left, circle));

		currentPath.removeLastOccurrence(n);
		return circle;
	}

	PathBasedCircleTreeNode branchOnPath(Node n, Path neighbourPath, Node otherPathEndNode, LinkedList<Tuple<Path, PathBasedCircleTreeNode>> edgesList, LinkedList<GraphComponent> currentPath)
	{
		PathBasedCircleTreeNode c = findCircles_branch(otherPathEndNode, neighbourPath , currentPath);
		if(c == null)
			throw new IllegalComponentStateException();

		c.addToPath(neighbourPath);

		if(c.isComplete())
		{
			edgesList.add(new Tuple<Path, PathBasedCircleTreeNode>(neighbourPath, c));
			return null;
		}
		else
			return c;
	}

	public PathBasedCircleTreeNode findLeaf() {
		for(PathBasedCircleTreeNode n : nodes)
			if(n.neighbours.size() == 1)
				return n;
		return null;
	}
}
