package solving.newSolver;

import java.util.ArrayList;

import main.Graph;
import main.Node;
import main.ThreeCycle;
import reductionRules.Reduction;
import result.DeletedEdgeEntry;
import result.ResultList;
import solving.BranchSolverStep;
import solving.GraphCopy;
import Utils.BranchingIterator;
import Utils.GraphUtils;

public class BSS1_ThreeCycles extends BranchSolverStep {

	public BSS1_ThreeCycles() {
	}

	@Override
	public int start(Graph g, int k, int maxK, ResultList resultList) {
		return branch(g, k, maxK, g.getThreeCycleBranchingIterator(), resultList);
	}

	private int branch(Graph g, int k, int maxK, BranchingIterator<ThreeCycle> iterator, ResultList resultList)
	{
		if(k > maxK)
			return -1;

		ThreeCycle cycle = null;
		int count = 0;
		while(iterator.hasNext())
		{
			count++;
			ThreeCycle c = iterator.next();
			if(c.isValid())
			{
				cycle = c;
				break;
			}
		}
		if(cycle == null)
		{
			iterator.rewind(count);
			return goToNextStep(g, step2Tokens, k, maxK, resultList);
		}
		recsteps++;

		//if(!cycle.isValid())
		//	throw new IllegalComponentStateException();

		int bestBranchK = Integer.MAX_VALUE;
		ResultList bestBranchResult = null;

		ResultList currentResult = new ResultList();
		int branchRes = branchOnEdge(cycle.n1, cycle.n2, g, k, bestBranchK, iterator, currentResult);
		if(branchRes != -1 && branchRes < bestBranchK)
		{
			bestBranchK = branchRes;
			bestBranchResult = currentResult;
		}

		currentResult = new ResultList();
		branchRes = branchOnEdge(cycle.n2, cycle.n3, g, k, bestBranchK, iterator, currentResult);
		if(branchRes != -1 && branchRes < bestBranchK)
		{
			bestBranchK = branchRes;
			bestBranchResult = currentResult;
		}

		currentResult = new ResultList();
		branchRes = branchOnEdge(cycle.n3, cycle.n1, g, k, bestBranchK, iterator, currentResult);
		if(branchRes != -1 && branchRes < bestBranchK)
		{
			bestBranchK = branchRes;
			bestBranchResult = currentResult;
		}
		iterator.rewind(count);

		if(bestBranchResult != null)
			resultList.addAll(bestBranchResult);
		return bestBranchK==Integer.MAX_VALUE? -1 : bestBranchK;
	}

	private int branchOnEdge(Node n1, Node n2, Graph g, int k, int maxK, BranchingIterator<ThreeCycle> iterator, ResultList resultList )
	{
		// TODO: the Graph g does not know about this edge removal, destroying
		// (possibly multiple) ThreeCycles. Its ThreeCycles will be out of date and
		// have to be rechecked! There must be a more efficient way!
		GraphUtils.removeEdge(n1, n2);
		boolean prevValue1 = n1.isConsistent();
		boolean prevValue2 = n2.isConsistent();
		n1.setConsistent(false);
		n2.setConsistent(false);
		DeletedEdgeEntry entry = new DeletedEdgeEntry(false, n1.name, n2.name);

		int kResult = branch(g, k+1, maxK, iterator, resultList);
		if(kResult != -1)
			resultList.add(entry);

		GraphUtils.addEdge(n1, n2);
		n1.setConsistent(prevValue1);
		n2.setConsistent(prevValue2);
		return kResult;
	}

	/*@Override
	protected int onLeaveStep(Graph g, ResultList resultList) {
		g.markCyclicNodes();
		int reduced = Reduction.applyTreeReductionRules(g, resultList);
		reduced += Reduction.applyPathReductionRules(g, resultList);
		return reduced;
	}*/


	@Override
	public String getName() {
		return "Branch solver step 1: Cycles";
	}

	@Override
	protected int goToNextStep(Graph g, BranchSolverStep defaultStep, int k, int maxK, ResultList resultList) {
		GraphCopy copy = new GraphCopy(g);
		// TODO: why reiterate over the copy, don't we know EXACTLY what vertices in
		// g are cyclic? Can't we just copy this information along with the nodes?

		//at this point there might be a lot of inconsistency as the value isCyclic is not updated after destroying 3-cycles
		ResultList reductionResult = new ResultList();

		//		GraphViewing.showGraphAndWaitForClose(g, "");

		ArrayList<Node> hooks = new ArrayList<Node>();
		for(Node n : copy.nodes)
		{
			if(!n.isConsistent())
			{
				Node hook = SolvingUtils.validateCyclicAndFindHook(n, null);
				if(hook != null)
					hooks.add(hook);
				n.setConsistent(true);
			}
		}
		int reduced = SolvingUtils.applyReductionForHooks(copy, hooks, reductionResult);

		copy.validateConsistency();

		int kResult = step2Tokens.start(copy, k+reduced, maxK, resultList);
		if(kResult != -1)
		{
			resultList.addAll(reductionResult);
		}
		return kResult;
	}


}
