package solving.newSolver;

import java.awt.IllegalComponentStateException;
import java.util.ArrayList;

import main.Degree2Path;
import main.Graph;
import main.Node;
import main.Token;
import reductionRules.Reduction;
import reductionRules.path.PathRedLPYSet;
import result.DeletedEdgeEntry;
import result.ResultList;
import solving.BranchSolverStep;
import solving.GraphCopy;
import Utils.GraphUtils;
import Utils.MarkingIterator;
import Utils.Tuple;

public class BSS2_Tokens extends BranchSolverStep{

	public BSS2_Tokens() {
	}


	@Override
	public int start(Graph g, int k, int maxK, ResultList resultList) 
	{
		g.validateConsistency();

		return branch(g, k, maxK, resultList);

	}

	private int branch(Graph g, int k, int maxK, ResultList resultList) 
	{
		g.validateConsistency();
		if(k > maxK)
			return -1;
		Degree2Path path = findNonBridgeDegree2PathWithToken(g);
		if(path == null)
			return goToNextStep(g, step3Claws, k, maxK, resultList);

		recsteps++;

		Token t = path.tokens[0];//TODO find the best token

		int bestBranchK = Integer.MAX_VALUE;
		ResultList bestBranchResult = null;

		int firstEdge = t.edges[0]==-1 ? t.edges[1] : t.edges[0];
		ResultList currentResult = new ResultList();
		int branchRes = copyAndBranchOnEdge(path.getPathVertex(firstEdge-1), path.getPathVertex(firstEdge), path, g, k, bestBranchK, currentResult);
		if(branchRes != -1 && branchRes < bestBranchK)
		{
			bestBranchK = branchRes;
			bestBranchResult = currentResult;
		}

		int lastEdge = t.edges[t.edges.length-1]==-1 ? t.edges[t.edges.length-2] : t.edges[t.edges.length-1];
		currentResult = new ResultList();
		branchRes = copyAndBranchOnEdge(path.getPathVertex(lastEdge-1), path.getPathVertex(lastEdge), path, g, k, bestBranchK, currentResult);
		if(branchRes != -1 && branchRes < bestBranchK)
		{
			bestBranchK = branchRes;
			bestBranchResult = currentResult;
		}

		if(t.edges[0] == -1)
		{
			currentResult = new ResultList();
			branchRes = copyAndbranchOnOuterPathEdges(path.getStart(), path.getPathVertex(1), path.getEnd(), path, g, k, bestBranchK, currentResult);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}
		}
		if(t.edges[t.edges.length-1] == -1)
		{
			currentResult = new ResultList();
			branchRes = copyAndbranchOnOuterPathEdges(path.getEnd(), path.getPathVertex(path.pathNodes.size()-2), path.getStart(), path, g, k, bestBranchK, currentResult);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}
		}

		if(bestBranchResult != null)
			resultList.addAll(bestBranchResult);
		
		return bestBranchK==Integer.MAX_VALUE? -1 : bestBranchK;
	}

	private int copyAndBranchOnEdge(Node n1, Node n2, Degree2Path path, Graph g, int k, int maxK, ResultList resultList)
	{
		GraphCopy copy = new GraphCopy(g);
		Node n1Copy = copy.getNodeClone(n1);
		Node n2Copy = copy.getNodeClone(n2);
		copy.validateConsistency();
		GraphUtils.removeEdge(n1Copy, n2Copy);
		DeletedEdgeEntry entry = new DeletedEdgeEntry(false, n1Copy.name, n2Copy.name);

		int reduced = 1;//we just removed one edge here
		makeInnerPathNodeAcyclic(path, copy);

		ResultList reductionResults = new ResultList();
		if(!path.isSimpleCircle())
		{
			ArrayList<Node> hooks = new ArrayList<Node>();
			if( path.getStart() == path.getEnd() ) 
				hooks.add(SolvingUtils.validateCyclicAndFindHook(copy.getNodeClone(path.getStart()), null)); 
			else
			{
				hooks.add(copy.getNodeClone(path.getStart()));
				hooks.add(copy.getNodeClone(path.getEnd())); 
			}

			reduced += SolvingUtils.applyReductionForHooks(copy, hooks, reductionResults);
		}
		else
			reduced += Reduction.applyTreeReductionRules(copy, GraphUtils.findLeaf(copy.nodes), null, reductionResults);

		copy.validateConsistency();
		int branchRes = branch(copy, k+reduced, maxK, resultList);
		if(branchRes != -1)
		{
			resultList.add(entry);

			//by branching deleted edge already contained in reduced, therefore subtract it
			resultList.addAll(reductionResults);
		}

		return branchRes;
	}

	private void makeInnerPathNodeAcyclic(Degree2Path path, GraphCopy copy)
	{
		int start = path.isSimpleCircle()? 0 : 1;
		int end = path.isSimpleCircle() ? path.pathNodes.size() : path.pathNodes.size()-1;
		for (int i = start; i < end; i++) 
		{
			Node pathNodeCopy = copy.getNodeClone(path.pathNodes.get(i));
			pathNodeCopy.setNonCyclic();
		}
	}

	private int copyAndbranchOnOuterPathEdges(Node pathEnd, Node lastInnerPathVertex, Node otherPathEnd, Degree2Path path, Graph g, int k, int maxK, ResultList resultList) 
	{
		g.validateConsistency();
		GraphCopy copy = new GraphCopy(g);
		Node pathEndCopy = copy.getNodeClone(pathEnd);
		Node lastInnerPathVertexCopy = copy.getNodeClone(lastInnerPathVertex);
		Node otherPathEndCopy = copy.getNodeClone(GraphUtils.getOtherEndNode(path, pathEnd));
		if(path.getStart() != path.getEnd())
			makeInnerPathNodeAcyclic(path, copy);
		pathEndCopy.setNonCyclic();
		return branchOnOuterPathEdges(pathEndCopy, lastInnerPathVertexCopy, otherPathEndCopy, copy, k, maxK, resultList);
	}

	private int branchOnOuterPathEdges(Node pathEnd, Node lastInnerPathVertex, Node otherPathEnd, Graph g, int k, int maxK, ResultList resultList) 
	{
		ArrayList<Node> outerCyclicNeighbours = new ArrayList<Node>();

		for(Node n : pathEnd.nonCyclicNeighbours)
		{
			if(n.getNeighbourCount() <= 2)//in this case (pathEnd, n) is a  non relevant a bridge
				return -1;
			else
				outerCyclicNeighbours.add(n);
		}
		for(Node n : pathEnd.cyclicNeighbours)
		{
			if(n != lastInnerPathVertex)
				outerCyclicNeighbours.add(n);
		}
		int reduced = outerCyclicNeighbours.size();
		ResultList tempResult = new ResultList();


		ArrayList<Node> hooks = new ArrayList<Node>(outerCyclicNeighbours.size()+1);
		
		for(Node n : outerCyclicNeighbours)
		{
			GraphUtils.removeEdge(pathEnd, n);
			tempResult.add(false, pathEnd, n);

			Node hook = SolvingUtils.validateCyclicAndFindHook(n, null);
			if(hook != null)
				hooks.add(hook);
		}
		hooks.add(otherPathEnd);
		reduced += SolvingUtils.applyReductionForHooks(g, hooks, tempResult);


		int branchRes = branch(g, k+reduced, maxK, resultList);

		if(branchRes != -1)
			resultList.addAll(tempResult);

		return branchRes;

	}

	/*private int reduceDestroyedPathsEnd_(Graph g, Node pathsEnd, ResultList resultList)
	{
		int reduced = 0;
		if(pathsEnd.isCyclic)
		{
			reduced += SolvingUtils.findHookAndApplyReduction(g, pathsEnd, null, resultList);
		}
		else{
			Node leaf = GraphUtils.findLeaf(GraphUtils.expand(pathsEnd, ExpandMode.nonCyclic));
			reduced += Reduction.applyTreeReductionRules(g, leaf, null, resultList);
		}
		return reduced;
	}*/

	//	private int findPathAndApplyReduction(Graph g, Node startNode, ResultList resultList)
	//	{
	//		Degree2Path path = Degree2Path.getMaximumDegree2Path(startNode);
	//		return Reduction.applyPathReductionRules(g, path, resultList);
	//	}

	private Degree2Path findNonBridgeDegree2PathWithToken(Graph g)
	{
		MarkingIterator<Node> nodeIterator = new MarkingIterator<Node>(g.nodes);
		while(nodeIterator.hasNext())
		{
			Node n = nodeIterator.next();
			if(n.isCyclic && n.cyclicNeighbours.size()==2 && PathRedLPYSet.getP2Count(n)>0)
			{
				Degree2Path newPath = Degree2Path.getMaximumDegree2Path(n, g);
				nodeIterator.markAll(newPath.pathNodes);
				
				if(newPath.getStart() == newPath.getEnd())
					continue;//choose next path
				newPath.findAndSetTokens();
				if(newPath.tokens.length == 0)
				{
					newPath.findAndSetTokens();
					throw new IllegalComponentStateException();
				}
				Node start = newPath.getStart();
				Node second = newPath.getInnerPathVertex(1);//TODO use GraphBBridgesInfo
				if(GraphUtils.hasNonCrossingCyclicPath(g, start, newPath.getEnd(), new Tuple<Node, Node>(start, second)))
					return newPath;
			}
		}
		return null;
	}

	@Override
	public String getName() {
		return "Branch Solver Step 2: Tokens";
	}


	@Override
	protected int goToNextStep(Graph g, BranchSolverStep defaultStep, int k, int maxK, ResultList resultList) {
		if(g.hasThreeCycles())
			return step1ThreeCycles.start(g, k, maxK, resultList);
		else
			return step3Claws.start(g, k, maxK, resultList);
	}
}
