package solving.newSolver;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Stack;

import main.Graph;
import main.Node;
import main.Start;
import reductionRules.path.PathRedLPYSet;
import result.DeletedEdgeEntry;
import result.ResultList;
import solving.BranchSolverStep;
import solving.GraphBBridgesInfo;
import solving.GraphCopy;
import visualization.GraphComponent;
import visualization.Graph.GraphViewing;
import Utils.ExpandMode;
import Utils.GraphUtils;
import Utils.Tuple;

public class BSS36_Claws extends BranchSolverStep 
{
	public BSS36_Claws() 
	{
	}

	@Override
	public int start(Graph g, int k, int maxK, ResultList resultList) {
		return branch(g, k, maxK, resultList);
	}

	private int branch(Graph g, int k, int maxK, ResultList resultList)
	{
		//		GraphViewing.showGraphAndWaitForClose(g, "");
		if(k > maxK)
			return -1;
		CycleClaw cw = findCycleClaw(g);
		
		if(cw == null)
			return goToNextStep(g, finisher, k, maxK, resultList);

		recsteps++;

		int bestBranchK = Integer.MAX_VALUE;
		ResultList bestBranchResult = null;

		if(cw.w1.length == 1)//CLAW0
		{
			ResultList currentResult = new ResultList();
			int branchRes = copyAndBranchOnTwoEdges(cw.v, cw.v1, cw.v, cw.v2, currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			currentResult = new ResultList();
			branchRes = copyAndBranchOnSingleEdge(cw.v1, cw.w1[0], currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			currentResult = new ResultList();
			branchRes = copyAndBranchOnSingleEdge(cw.v2, cw.w2[0], currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			currentResult = new ResultList();
			branchRes = copyAndBranchOnSingleEdge(cw.v3, cw.w3[0], currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}
		}
		else if(cw.w2.length == 1)//CLAW1
		{
			ResultList currentResult = new ResultList();
			int branchRes = copyAndBranchOnSingleEdge(cw.v, cw.v1, currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			currentResult = new ResultList();
			branchRes = copyAndBranchOnTwoEdges(cw.v, cw.v2, cw.v, cw.v3, currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			if(!isABridge(cw.v1, cw.w1[0]) && !isABridge(cw.v1, cw.w1[1])) 
			{
				currentResult = new ResultList();
				branchRes = copyAndBranchOnTwoEdges(cw.v1, cw.w1[0], cw.v1, cw.w1[1], currentResult, g, k, bestBranchK);
				if(branchRes != -1 && branchRes < bestBranchK)
				{
					bestBranchK = branchRes;
					bestBranchResult = currentResult;
				}
			}

			currentResult = new ResultList();
			branchRes = copyAndBranchOnSingleEdge(cw.v2, cw.w2[0], currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			currentResult = new ResultList();
			branchRes = copyAndBranchOnSingleEdge(cw.v3, cw.w3[0], currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}
		}
		else if(cw.w3.length == 1)//CLAW2
		{
			ResultList currentResult = new ResultList();
			int branchRes = copyAndBranchOnSingleEdge(cw.v, cw.v1, currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			currentResult = new ResultList();
			branchRes = copyAndBranchOnSingleEdge(cw.v, cw.v2, currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			if(!isABridge(cw.v1, cw.w1[0]) && !isABridge(cw.v1, cw.w1[1]))
			{
				currentResult = new ResultList();
				branchRes = copyAndBranchOnTwoEdges(cw.v1, cw.w1[0], cw.v1, cw.w1[1], currentResult, g, k, bestBranchK);
				if(branchRes != -1 && branchRes < bestBranchK)
				{
					bestBranchK = branchRes;
					bestBranchResult = currentResult;
				}
			}

			if(!isABridge(cw.v2, cw.w2[0]) && !isABridge(cw.v2, cw.w2[1]))
			{
				currentResult = new ResultList();
				branchRes = copyAndBranchOnTwoEdges(cw.v2, cw.w2[0], cw.v2, cw.w2[1], currentResult, g, k, bestBranchK);
				if(branchRes != -1 && branchRes < bestBranchK)
				{
					bestBranchK = branchRes;
					bestBranchResult = currentResult;
				}
			}

			currentResult = new ResultList();
			branchRes = copyAndBranchOnSingleEdge(cw.v3, cw.w3[0], currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}
		}
		else//CLAW3
		{
			ResultList currentResult = new ResultList();
			int branchRes = copyAndBranchOnSingleEdge(cw.v, cw.v1, currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			currentResult = new ResultList();
			branchRes = copyAndBranchOnSingleEdge(cw.v, cw.v2, currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			currentResult = new ResultList();
			branchRes = copyAndBranchOnSingleEdge(cw.v, cw.v3, currentResult, g, k, bestBranchK);
			if(branchRes != -1 && branchRes < bestBranchK)
			{
				bestBranchK = branchRes;
				bestBranchResult = currentResult;
			}

			if(!isABridge(cw.v1, cw.w1[0]) && !isABridge(cw.v1, cw.w1[1]))
			{
				currentResult = new ResultList();
				branchRes = copyAndBranchOnTwoEdges(cw.v1, cw.w1[0], cw.v1, cw.w1[1], currentResult, g, k, bestBranchK);
				if(branchRes != -1 && branchRes < bestBranchK)
				{
					bestBranchK = branchRes;
					bestBranchResult = currentResult;
				}
			}

			if(!isABridge(cw.v2, cw.w2[0]) && !isABridge(cw.v2, cw.w2[1]))
			{
				currentResult = new ResultList();
				branchRes = copyAndBranchOnTwoEdges(cw.v2, cw.w2[0], cw.v2, cw.w2[1], currentResult, g, k, bestBranchK);
				if(branchRes != -1 && branchRes < bestBranchK)
				{
					bestBranchK = branchRes;
					bestBranchResult = currentResult;
				}
			}
			if(!isABridge(cw.v3, cw.w3[0]) && !isABridge(cw.v3, cw.w3[1]))
			{
				currentResult = new ResultList();
				branchRes = copyAndBranchOnTwoEdges(cw.v3, cw.w3[0], cw.v3, cw.w3[1], currentResult, g, k, bestBranchK);
				if(branchRes != -1 && branchRes < bestBranchK)
				{
					bestBranchK = branchRes;
					bestBranchResult = currentResult;
				}
			}
		}
		if(bestBranchResult != null)
			resultList.addAll(bestBranchResult);
		return bestBranchK==Integer.MAX_VALUE? -1 : bestBranchK;
	}

	private int copyAndBranchOnSingleEdge(Node n1, Node n2, ResultList resultList, Graph g, int k, int maxK)
	{
		if(isNonRelevantABridge(n1, n2))
			return -1;
		GraphCopy copy = new GraphCopy(g);
		Node n1Copy = copy.getNodeClone(n1);
		Node n2Copy = copy.getNodeClone(n2);
		GraphUtils.removeEdge(n1Copy, n2Copy);
		DeletedEdgeEntry entry = new DeletedEdgeEntry(false, n1Copy.name, n2Copy.name);

		ResultList reducedList = new ResultList();
		int reduced = 1;


		ArrayList<Node> hooks = new ArrayList<Node>();
		Node hook = SolvingUtils.validateCyclicAndFindHook(n1Copy, n2Copy);
		if(hook != null)
			hooks.add(hook);
		hook = SolvingUtils.validateCyclicAndFindHook(n2Copy, n1Copy);
		if(hook != null)
			hooks.add(hook);

		reduced += SolvingUtils.applyReductionForHooks(copy, hooks, reducedList);


		int branchRes = goToNextStep(copy, step2Tokens, k+reduced, maxK, resultList);
		if(branchRes != -1)
		{
			resultList.add(entry);

			//by branching deleted edge already contained in reduced, therefore subtract it
			resultList.addAll(reducedList);
		}
		return branchRes;
	}

	private int copyAndBranchOnTwoEdges(Node n1, Node n2, Node m1, Node m2, ResultList resultList, Graph g, int k, int maxK)
	{
		if(isNonRelevantABridge(n1, n2))
			return -1;
		else if(isNonRelevantABridge(m1, m2))
			return -1;
		g.validateConsistency();
		GraphCopy copy = new GraphCopy(g);
		copy.validateConsistency();
		Node n1Copy = copy.getNodeClone(n1);
		Node n2Copy = copy.getNodeClone(n2);
		Node m1Copy = copy.getNodeClone(m1);
		Node m2Copy = copy.getNodeClone(m2);
		GraphUtils.removeEdge(n1Copy, n2Copy);
		GraphUtils.removeEdge(m1Copy, m2Copy);
		DeletedEdgeEntry entry1 = new DeletedEdgeEntry(false, n1Copy.name, n2Copy.name);
		DeletedEdgeEntry entry2 = new DeletedEdgeEntry(false, m1Copy.name, m2Copy.name);


		ResultList reducedList = new ResultList();
		int reduced = 2;

		//		Node[] hooks = new Node[]{
		//				SolvingUtils.validateCyclicAndFindHook(n1Copy, n2Copy),
		//				SolvingUtils.validateCyclicAndFindHook(n2Copy, n1Copy),
		//				SolvingUtils.validateCyclicAndFindHook(m1Copy, m2Copy),
		//				SolvingUtils.validateCyclicAndFindHook(m2Copy, m1Copy)
		//		};

		ArrayList<Node> hooks = new ArrayList<Node>();
		Node hook = SolvingUtils.validateCyclicAndFindHook(n1Copy, n2Copy);
		if(hook != null)
			hooks.add(hook);
		hook = SolvingUtils.validateCyclicAndFindHook(n2Copy, n1Copy);
		if(hook != null)
			hooks.add(hook);
		hook = SolvingUtils.validateCyclicAndFindHook(m1Copy, m2Copy);
		if(hook != null)
			hooks.add(hook);
		hook = SolvingUtils.validateCyclicAndFindHook(m2Copy, m1Copy);
		if(hook != null)
			hooks.add(hook);

		reduced += SolvingUtils.applyReductionForHooks(copy, hooks, reducedList);

		copy.validateConsistency();

		int branchRes = goToNextStep(copy, step2Tokens, k+reduced, maxK, resultList);
		if(branchRes != -1)
		{
			resultList.add(entry1);
			resultList.add(entry2);

			//by branching deleted edges already contained in reduced, therefore subtract them
			resultList.addAll(reducedList);
		}
		return branchRes;
	}



	private boolean isABridge(Node n1, Node n2)
	{
		return !n1.isCyclic || !n2.isCyclic;
	}

	private boolean isNonRelevantABridge(Node n1, Node n2)
	{
		//return (!n1.isCyclic || !n1.Y.contains(n2)) && (!n2.isCyclic || !n2.Y.contains(n1));
//		return (!n1.isCyclic && !n2.isCyclic) ||
//				((!n1.isCyclic && PathRedLPYSet.getYGraph(n2) != n1) &&
//						(!n2.isCyclic && PathRedLPYSet.getYGraph(n1) != n2));
		return isABridge(n1, n2) && !isRelevantABridge(n1, n2);
	}

	private boolean isRelevantABridge(Node n1, Node n2)
	{
		return (n1.isCyclic && PathRedLPYSet.getYGraph(n1) == n2) 
				|| (n2.isCyclic && PathRedLPYSet.getYGraph(n2) == n1);
	}

	private CycleClaw findCycleClaw(Graph g)
	{
		g.validateConsistency();
		GraphBBridgesInfo bridgesInfo = new GraphBBridgesInfo(g);

		for(Node v : g.nodes)
		{
			if(!v.isCyclic || v.getNeighbourCount() < 3)
				continue;
			if(bridgesInfo.isBBridgeNode(v))
				continue;

			Node[] vi = new Node[3];
			int i = 0;
			for(Node n : v.nonCyclicNeighbours)
			{
				if(i >= 3)
					break;
				if(n.getNeighbourCount() >= 2 && !isRelevantABridge(n, v))
					vi[i++] = n;
			}
			for(Node n : v.cyclicNeighbours)
			{
				if(i >= 3)
					break;
				if(/*n.getNeighbourCount() >= 2 && */!bridgesInfo.isAdjacentToBBridge(n))
					vi[i++] = n;
			}
			if(vi[2] == null)
				continue;

			Arrays.sort(vi, invertingNodeNeighbourCountComparator);



			//### Observation 3
			boolean error = false;
			if(vi[0].getNeighbourCount() == 2)
			{
				if(!isABridge(v, vi[0]))
				{
					Tuple<Node, Node> notToCross1 = new Tuple<Node, Node>(v, vi[0]);
					Tuple<Node, Node> notToCross2 = new Tuple<Node, Node>(v, vi[1]);
					if(!GraphUtils.hasNonCrossingCyclicPath(g, v, vi[0], notToCross1, notToCross2))
					{//swap
						Node temp = vi[1];
						vi[1] = vi[2];
						vi[2] = temp;
					}
				}
			}
			else if(!isABridge(v, vi[1]) && vi[1].getNeighbourCount() == 2)
			{
				Tuple<Node, Node> notToCross1 = new Tuple<Node, Node>(v, vi[1]);
				Tuple<Node, Node> notToCross2 = new Tuple<Node, Node>(v, vi[2]);
				if(!GraphUtils.hasNonCrossingCyclicPath(g, v, vi[1], notToCross1, notToCross2))
				{//swap
					Node vi1_alt = vi[1];
					for(Node neighbour : v.cyclicNeighbours)
					{
						if(neighbour != vi[0] && neighbour != vi[1] && neighbour != vi[2])
						{
							vi[1] = neighbour;
							break;
						}
					}
					if(Start.debugMode && vi1_alt == vi[1])
					{
						error = true;
						//throw new IllegalComponentStateException();
					}
				}
			}
			//### Observation 3 ### end

			Node[][] w = new Node[3][]; 

			for(i = 0; i < 3; i++)
			{
				if(vi[i].getNeighbourCount() == 2){
					w[i] = new Node[]{GraphUtils.getOtherNeighbour(vi[i], v, ExpandMode.both)};
				} else
				{
					Node a = getNonCrossingPathsLastNode(v, vi[i], new Tuple<Node, Node>(v, vi[i]));
					w[i] = new Node[]{a, null};

					int j = a==null?0:1;
					for(Node n : vi[i].cyclicNeighbours)
					{
						if(j >= 2)
							break;
						if(n != v && n != a)
							w[i][j++] = n;
					}

					if(j < 2)
						for(Node n : vi[i].nonCyclicNeighbours)
						{
							if(j >= 2)
								break;
							//if(n != w[i][0])
							w[i][j++] = n;
						}
				}
			}

			CycleClaw cc = new CycleClaw(v, vi[0], vi[1], vi[2], w[0], w[1], w[2]);
			if(error)
				GraphViewing.showGraphAndWaitForClose(g, "error", cc);
			return cc;
		}
		return null;
	}

	private Comparator<Node> invertingNodeNeighbourCountComparator = new Comparator<Node>() {
		@Override
		public int compare(Node n1, Node n2) {
			//return Integer.compare(n2.getNeighbourCount(), n1.getNeighbourCount());
			//return (n2.getNeighbourCount()-n1.getNeighbourCount());
			int n1Deg = n1.getNeighbourCount();
			int n2Deg = n2.getNeighbourCount();
			return (n1Deg > n2Deg) ? -1 : ((n1Deg == n2Deg) ? 0 : 1);

		}
	};


	/**
	 * 
	 * @param from
	 * @param to
	 * @param edgeNotToCross
	 * @return
	 */
	public static Node getNonCrossingPathsLastNode(Node from, Node to, Tuple<Node, Node> edgeNotToCross)
	{ 
		HashSet<Node> visited = new HashSet<Node>();

		Stack<Node> toExpand = new Stack<Node>();
		toExpand.push(from);

		while(toExpand.size() > 0)
		{
			Node n = toExpand.pop();
			visited.add(n);
			boolean nIsInbannedEdge = edgeNotToCross.contains(n);
			for(Node neighbour : n.cyclicNeighbours)
			{
				if(nIsInbannedEdge && edgeNotToCross.contains(neighbour))
					continue;
				if(neighbour == to)
					return n;
				if(!visited.contains(neighbour))
					toExpand.add(neighbour);
			}
		}
		return null;
	}


	@Override
	public String getName() {
		// TODO Auto-generated method stub
		return "Branch solver steps 3-6: Claws";
	}

	private class CycleClaw implements GraphComponent
	{
		Node v;
		Node v1, v2, v3;
		Node[] w1, w2, w3;

		public CycleClaw(Node v, Node v1, Node v2, Node v3,	Node[] w1, Node[] w2, Node[] w3) {
			this.v = v;
			this.v1 = v1;
			this.v2 = v2;
			this.v3 = v3;

			this.w1 = w1;
			this.w2 = w2;
			this.w3 = w3;
		}

		@Override
		public boolean contains(Node n) {
			return v == n
					|| v1 == n || v2 == n || v3 == n
					|| w1[0] == n || w2[0] == n || w3[0] == n
					|| (w1.length == 2 && w1[1] == n)
					|| (w2.length == 2 && w2[1] == n)
					|| (w3.length == 2 && w3[1] == n);
		}
	}

	@Override
	protected int goToNextStep(Graph g, BranchSolverStep defaultStep, int k, int maxK, ResultList resultList) {
		if(g.hasThreeCycles())
			return step1ThreeCycles.start(g, k, maxK, resultList);
		else
			return defaultStep.start(g, k, maxK, resultList);
	}
}
