package solving.newSolver;

import java.awt.IllegalComponentStateException;
import java.util.ArrayList;
import java.util.HashSet;

import main.Graph;
import main.Node;
import main.Start;
import reductionRules.Reduction;
import reductionRules.path.PathRedLPYSet;
import result.ResultList;
import solving.BranchSolverStep;
import solving.GraphCopy;
import solving.circlesTree.ComponentTreeNode;
import solving.circlesTree.ComponentsTree;
import visualization.Graph.GraphViewing;
import Utils.ConnCompIterator;
import Utils.ExpandMode;
import Utils.GraphUtils;

public class NewBranchFinisher extends BranchSolverStep 
{
	public NewBranchFinisher() {
	}

	@Override
	public int start(Graph g, int k, int maxK, ResultList resultList) 
	{
		GraphCopy copy = new GraphCopy(g);
		copy.validateConsistency();


		ResultList reductionResult = new ResultList();

		int reduced = componentsTreeReduction(copy, reductionResult);
		if(k+reduced > maxK)
			return -1;
		

		if(copy.nodes.size() > 0)
			reduced += Reduction.applyTreeReductionRules(copy, reductionResult);

		if(k+reduced > maxK)
			return -1;

		Reduction.removeCaterpillars(copy);
		if(copy.nodes.size()>0)
		{
			int branchRes = goToNextStep(copy, step2Tokens, k+reduced, maxK, resultList);
			if(branchRes != -1)
			{
				resultList.addAll(reductionResult);
			}
			return branchRes;
		}
		else
		{
			//GraphViewing.showGraphAndWaitForClose(g, this.getName());
			resultList.addAll(reductionResult);
			return k+reduced;
		}
	}

	@Override
	public String getName() {
		return "branch finisher";
	}

	private int componentsTreeReduction(Graph g, ResultList resultList)
	{
		int reduced = 0;
		ConnCompIterator compIterator = new ConnCompIterator(g);
		while(compIterator.hasNext())
		{
			HashSet<Node> comp = compIterator.next();

			//GraphViewing.showGraphAndWaitForClose(g, "");
			ComponentsTree ctree = new ComponentsTree(g, comp);

			if(ctree.components.size() == 1)
			{
				reduced += handleLastComponent(ctree.components.get(0), resultList);
			}
			else
			{
				ArrayList<ComponentTreeNode> leaves = ctree.findLeaves();
				ArrayList<Node> hooks = new ArrayList<Node>(leaves.size());
				ArrayList<Node> hookStarterNodes = new ArrayList<Node>(leaves.size());
				int i = 0;
				for(ComponentTreeNode leaf : leaves)
				{
					Node anchor = leaf.anchorNodes.iterator().next();
					Node outerNode = leaf.getFirstOuterCircleNodeForAnchor(anchor);
					Node nLeft = null;
					Node nRight = null;
					for(Node neighbour : anchor.cyclicNeighbours)
					{
						if(neighbour != outerNode)
						{
							if(nLeft == null)
								nLeft = neighbour;
							else if(nRight == null)
								nRight = neighbour;
							else 
								throw new IllegalComponentStateException();
						}
					}
					if(nRight == null)
						throw new IllegalComponentStateException();

					//### applying reduction rule here
					Node edgeToDeleteN1 = null;
					Node edgeToDeleteN2 = null;
//					
					Node relevantNeighbour = PathRedLPYSet.hasP2(nLeft) ? nLeft : nRight;
					
					if(PathRedLPYSet.hasP2(relevantNeighbour))//if neighbour is dirty
					{
						Node relevantNeighbour2 = GraphUtils.getOtherNeighbour(relevantNeighbour, anchor, ExpandMode.cyclic);
						if(PathRedLPYSet.hasLeaf(relevantNeighbour2))//if crossing
						{
							edgeToDeleteN1 = relevantNeighbour;
							edgeToDeleteN2 = relevantNeighbour2;
						}
						else
						{ 
							edgeToDeleteN1 = relevantNeighbour2;
							edgeToDeleteN2 = GraphUtils.getOtherNeighbour(relevantNeighbour2, relevantNeighbour, ExpandMode.cyclic);
						}
					}
					else
					{
						edgeToDeleteN1 = anchor;
						edgeToDeleteN2 = relevantNeighbour;
					}
					resultList.add(true, edgeToDeleteN1, edgeToDeleteN2);
					GraphUtils.removeEdge(edgeToDeleteN1, edgeToDeleteN2);
					reduced++;

					hookStarterNodes.add(edgeToDeleteN1);	
					hookStarterNodes.add(edgeToDeleteN2);

					//reduced += ctree.onNodeDestroyed(leaf, resultList);
				}

				for(Node hookStart : hookStarterNodes)
				{
					Node hook = SolvingUtils.validateCyclicAndFindHook(hookStart, null);
					if(hook != null)
						hooks.add(hook);
				}

				reduced += SolvingUtils.applyReductionForHooks(g, hooks, resultList);
				//reduced += onLeafsDestroyed(g, leaves, resultList);
			}
			/*if(ctree.circles.size()==1)
			{
				ComponentTreeNode lastCircle = ctree.circles.get(0);
				if(lastCircle.anchorNodes.size()==0)
					reduced += handleLastCircle(ctree, lastCircle, resultList);
			}*/
			//	else if(ctree.circles.size()>1)		
			//		throw new IllegalComponentStateException();
		}
		return reduced;
	}


	private int handleLastComponent(ComponentTreeNode lastComp, ResultList resultList)
	{
		//		for (int i = 0; i < lastComp.nodes.size(); i++) {
		//			Node n = lastComp.nodes.get(i);
		//
		//			if(PathRedLPYSet.hasYGraph(n))
		//			{
		//				//find any neighbour node of n within this circle
		//				Node other = i == 0 ?  lastComp.nodes.get(1) :  lastComp.nodes.get(i-1);
		//				resultList.add(n, other);
		//				GraphUtils.removeEdge(n, other);
		//				return 1;
		//			}
		//		}
		//if no y graph found choose any edge
		Node n0 = lastComp.nodes.iterator().next();
		Node n1 = n0.cyclicNeighbours.iterator().next();
		resultList.add(true, n0, n1);
		GraphUtils.removeEdge(n0, n1);

		for(Node n : lastComp.nodes)
			n.setNonCyclic();
		return 1;
	}

	@Override
	protected int goToNextStep(Graph g, BranchSolverStep defaultStep, int k, int maxK, ResultList resultList) {
		//		GraphViewing.showGraphAndWaitForClose(g, "");
		g.validateConsistency();
		return defaultStep.start(g, k, maxK, resultList);
	}
}
