package solving.oldSolver.steps;

import java.util.HashSet;
import java.util.LinkedList;

import main.Graph;
import main.Node;
import main.Start;
import solving.BranchSolverStep_old;
import solving.TwoClaw;
import Utils.GraphUtils;

public class BSS1_SeparateCircles extends BranchSolverStep_old 
{
	public BSS1_SeparateCircles(BranchSolverStep_old nextStep) 
	{
		super(nextStep);
	}

	@Override
	public int start(Graph g, int k)
	{

		if(k < 0)
			return -1;

		TwoClaw tc = findTwoClaw(g);
		if(tc == null)//the given circlesTree should be null here
			return goToNextStep(g,k);

		int res = -1;
		if(-1 != (res = solveStep1_branchOnEdge(g, k, tc.c, tc.u1)))
			return res;
		if(-1 != (res = solveStep1_branchOnEdge(g, k, tc.u1, tc.u2)))
			return res;
		if(-1 != (res = solveStep1_branchOnEdge(g, k, tc.c, tc.v1)))
			return res;
		if(-1 != (res = solveStep1_branchOnEdge(g, k, tc.v1, tc.v2)))
			return res;
		if(-1 != (res = solveStep1_branchOnEdge(g, k, tc.c, tc.w1)))
			return res;
		if(-1 != (res = solveStep1_branchOnEdge(g, k, tc.w1, tc.w2)))
			return res;

		return -1;
	}
	private int solveStep1_branchOnEdge(Graph g, int k, Node n1, Node n2)
	{
		if(Start.useBranchTreeVisualization)
		{
			Edge e = new Edge(n1, n2);
			OnBeforeChange(g, e);
			GraphUtils.removeEdge(n1, n2);
			onEdgeDeleted(n1, n2);
			onAfterChange(g, e);
		}
		else
			GraphUtils.removeEdge(n1, n2);

		int res = -1;
		if(-1 != (res = start(g,k-1)))
		{
			onBranchReturn();
			return res;
		}
		GraphUtils.addEdge(n1, n2);
		onEdgeAdded(n1, n2);
		onBranchReturn();
		return -1;
	}

	private TwoClaw findTwoClaw(Graph g)
	{
		HashSet<Node> cycleNodes = new HashSet<Node>();
		setCycleGraphNodes(g.nodes.getFirst(), null, new LinkedList<Node>(), cycleNodes);

		for(Node n : cycleNodes)
		{
			LinkedList<Node> neighbours = null;
			if( n.cyclicNeighbours.size() >= 3 && null != (neighbours=getEnoughCyclicNeighbours(n, 3, cycleNodes)))
			{
				Node u1=null, u2=null, v1=null, v2=null, w1=null, w2=null;

				u1 = neighbours.poll();
				u2 = getOtherCyclicNeighbour(u1, n, cycleNodes);

				v1 = neighbours.poll();
				v2 = getOtherCyclicNeighbour(v1, n, cycleNodes);

				w1 = neighbours.poll();
				w2 = getOtherCyclicNeighbour(w1, n, cycleNodes);

				return new TwoClaw(n, u1, u2, v1, v2, w1, w2);
			}
		}
		return null;
	}

	private Node getOtherCyclicNeighbour(Node node, Node notToChoose, HashSet<Node> allowed)
	{
		for(Node n : node.cyclicNeighbours)
		{
			if(n!= notToChoose && allowed.contains(n))
				return n;
		}
		throw new IllegalArgumentException();
	}

	private LinkedList<Node> getEnoughCyclicNeighbours(Node toCheck, int minNeighbours, HashSet<Node> allowedNodes)
	{
		LinkedList<Node> nodes = new LinkedList<Node>();
		for(Node neighbour : toCheck.cyclicNeighbours)
		{
			if(allowedNodes.contains(neighbour))
			{
				nodes.add(neighbour);
				if(nodes.size() >= minNeighbours)
					return nodes;
			}
		}
		return null;
	}

	private Node setCycleGraphNodes(Node n, Node prevNode, LinkedList<Node> currentPath, HashSet<Node> cycleGraph)
	{
		if(currentPath.contains(n))
		{
			cycleGraph.add(n);
			return n;
		}
		if(cycleGraph.contains(n))
			return null;

		currentPath.addLast(n);

		Node returnVal = null;
		int returnValIndex = -1;
		for(Node neighbour : n.cyclicNeighbours)
		{
			//prevent goint backwards
			if(neighbour == prevNode)
				continue;

			Node cyclicNode = setCycleGraphNodes(neighbour, n, currentPath, cycleGraph);

			if(cyclicNode != null)
			{
				int cyclicNodeIndex = currentPath.indexOf(cyclicNode);
				if(returnVal == null)
				{
					returnVal = cyclicNode;
					returnValIndex = cyclicNodeIndex;
				}
				else if(cyclicNodeIndex < returnValIndex)
				{
					returnVal = cyclicNode;
					returnValIndex = cyclicNodeIndex;
				}
			}

		}
		if(returnVal == n)
			returnVal = null;

		if(returnVal != null)
			cycleGraph.add(n);

		currentPath.removeLastOccurrence(n);
		return returnVal;
	}
	@Override
	public String getName() {
		return "Branch-solver-step 1: Separate Circles";
	}
}
