package visualization.BranchingTree;

import java.awt.Dimension;
import java.awt.Paint;

import main.Node;

import org.apache.commons.collections15.Transformer;

import solving.BranchSolverStep_old;
import visualization.GraphComponent;
import visualization.GraphVisColors;
import Utils.Tuple;
import edu.uci.ics.jung.algorithms.layout.TreeLayout;
import edu.uci.ics.jung.graph.DelegateForest;
import edu.uci.ics.jung.graph.Forest;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ModalGraphMouse.Mode;

public class TreeViewer extends VisualizationViewer<BranchTreeNode, String> {

	private static final long serialVersionUID = 4376809249862669596L;
	private Forest<BranchTreeNode, String> forest;
	private NodeColorTransformer nodeColTrans;

	private BranchTreeNode baseNode;

	private static DelegateForest<BranchTreeNode, String> createSimpleTree(BranchTreeNode startNode)
	{
		DelegateForest<BranchTreeNode, String> forest = new DelegateForest<BranchTreeNode, String>();
		forest.addVertex(startNode);
		return forest;
	}
	public TreeViewer(BranchTreeNode startNode) {
		this(createSimpleTree(startNode));
		this.baseNode = startNode;
	}


	private TreeViewer(DelegateForest<BranchTreeNode, String> forest) {
		super(new TreeLayout<BranchTreeNode, String>(forest), new Dimension(600,600));

		this.forest = forest;
		nodeColTrans = new NodeColorTransformer();
		this.getRenderContext().setVertexFillPaintTransformer(nodeColTrans);

		DefaultModalGraphMouse<Node, Tuple<Node, Node>> graphMouse = new DefaultModalGraphMouse<Node, Tuple<Node, Node>>();
		graphMouse.setMode(Mode.PICKING);
		this.setGraphMouse(graphMouse);

	}


	int i = 0;
	public BranchTreeNode addNewNode(BranchTreeNode parent, GraphComponent component, BranchSolverStep_old currentStep)
	{
		BranchTreeNode node = new BranchTreeNode(component, currentStep);
		forest.addVertex(node);
		
		forest.addEdge(""+(i++), parent==null?baseNode : parent, node);
		nodeColTrans.setCurrentComponent(component);
		return node;
	}


	private class NodeColorTransformer implements Transformer<BranchTreeNode, Paint>
	{
		GraphComponent comp;

		protected void setCurrentComponent(GraphComponent comp)
		{
			this.comp = comp;
		}

		@Override
		public Paint transform(BranchTreeNode node) 
		{
			if(node.component == comp)
				return GraphVisColors.currentObject;
			else 
				return GraphVisColors.normal;
		}
	}
}
