package visualization.Graph;


import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import main.Graph;
import visualization.GraphChanger;
import visualization.GraphComponent;


public class GraphViewFrame extends JFrame
{
	public enum BreakPoint
	{
		now,
		nextRule,
		nextComponent,
		nextChange,
		noBreak
	}

	///boolean next;
	BreakPoint nextStop;
	GraphViewer graphViewer;
	GraphComponent currentFocus;

	JLabel ruleNameLabel;

	/**
	 * 
	 */
	private static final long serialVersionUID = -1959968224570858484L;

	public GraphViewFrame(Graph g)
	{
		super("");
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		Container pane = getContentPane();
		pane.setLayout(new BorderLayout());
		nextStop = BreakPoint.now;
		//next = false;

		pane.add(ruleNameLabel=new JLabel(" "), BorderLayout.NORTH);
		pane.add(graphViewer=new GraphViewer(g), BorderLayout.CENTER);

		JPanel eastPanel = new JPanel(new GridLayout(2, 1, 0, 10));
		eastPanel.add(new Legend());

		JPanel eastLowerPanel = new JPanel(new GridLayout(3, 1));
		JPanel EastLowerPanelCont = new JPanel();
		EastLowerPanelCont.add(eastLowerPanel);
		eastPanel.add(EastLowerPanelCont);


		JButton button = new JButton("Next Rule");
		addMouseClickListener(button, new MouseClickListener() {
			@Override
			public void mouseClicked(MouseEvent e) {
				nextRuleButtonClicked();
			}
		});
		eastLowerPanel.add(button);

		button = new JButton("Next Component");
		addMouseClickListener(button, new MouseClickListener() {
			@Override
			public void mouseClicked(MouseEvent e) {
				nextComponentButtonClicked();
			}
		});
		eastLowerPanel.add(button);

		button = new JButton("Next Change");
		addMouseClickListener(button, new MouseClickListener() {
			@Override
			public void mouseClicked(MouseEvent e) {
				nextChangeButtonClicked();
			}
		});
		eastLowerPanel.add(button);
		
		button = new JButton("To Finish");
		addMouseClickListener(button, new MouseClickListener() {
			@Override
			public void mouseClicked(MouseEvent e) {
				toFinishButtonClicked();
			}
		});
		eastLowerPanel.add(button);

		pane.add(eastPanel, BorderLayout.EAST);
		this.pack();
	}

	void drawChanges( GraphChanger c, GraphComponent d) 
	{
		currentFocus = d;
		ruleNameLabel.setText(c.getName());
		graphViewer.drawChanges(c, d);
	}

	synchronized void nextRuleButtonClicked()
	{
		nextStop = BreakPoint.nextRule;
		this.notifyAll();
	}
	synchronized void nextComponentButtonClicked()
	{
		nextStop = BreakPoint.nextComponent;
		this.notifyAll();
	}
	synchronized void nextChangeButtonClicked()
	{
		nextStop = BreakPoint.nextChange;
		this.notifyAll();
	}
	synchronized void toFinishButtonClicked()
	{
		nextStop = BreakPoint.nextChange;
		this.notifyAll();
	}

	private void addMouseClickListener(JButton button, final MouseClickListener mouseListener)
	{
		button.addMouseListener(new MouseListener() {
			@Override
			public void mouseReleased(MouseEvent e) {}
			@Override
			public void mousePressed(MouseEvent e) {}
			@Override
			public void mouseExited(MouseEvent e) {}
			@Override
			public void mouseEntered(MouseEvent e) {}

			@Override
			public void mouseClicked(MouseEvent e) {
				mouseListener.mouseClicked(e);
			}
		});
	}

	private interface MouseClickListener
	{
		void mouseClicked(MouseEvent e);
	}
}
