package visualization.Graph;

import org.apache.commons.collections15.Transformer;

import main.Graph;
import main.Node;
import visualization.GraphChanger;
import visualization.GraphComponent;
import Utils.Tuple;
import edu.uci.ics.jung.algorithms.layout.FRLayout;
import edu.uci.ics.jung.graph.UndirectedSparseGraph;
import edu.uci.ics.jung.graph.util.EdgeType;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.DefaultModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ModalGraphMouse.Mode;

public class GraphViewer extends VisualizationViewer<Node, Tuple<Node, Node>> {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8647977946391643441L;

	private static UndirectedSparseGraph<Node, Tuple<Node,Node>> convert(Graph g) 
	{
		UndirectedSparseGraph<Node, Tuple<Node,Node>> converted = new UndirectedSparseGraph<Node, Tuple<Node,Node>>();

		for(Node n : g.nodes)
			converted.addVertex(n);

		boolean[][] addedEdges = new boolean[g.initialNodesCount][g.initialNodesCount];
		for(Node n : g.nodes)
		{
			for(Node neighbour : n.cyclicNeighbours)
			{
				if(!addedEdges[n.index][neighbour.index])
				{
					converted.addEdge(new Tuple<Node,Node>(n, neighbour), n, neighbour, EdgeType.UNDIRECTED);
					addedEdges[neighbour.index][n.index] = addedEdges[n.index][neighbour.index] = true;
				}
			}
			for(Node neighbour : n.nonCyclicNeighbours)
			{
				if(!addedEdges[n.index][neighbour.index])
				{
					converted.addEdge(new Tuple<Node,Node>(n, neighbour), n, neighbour, EdgeType.UNDIRECTED);
					addedEdges[neighbour.index][n.index] = addedEdges[n.index][neighbour.index] = true;
				}
			}
		}
		return converted;
	}

	private final UndirectedSparseGraph<Node,Tuple<Node, Node>> drawnGraph;
	protected final NodeColorTransformer nodeColTransf;
	private final EdgeColorTransformer edgeColTransf;
	
	public GraphViewer(Graph g) 
	{
		this(convert(g));
	}

	public GraphViewer(UndirectedSparseGraph<Node,Tuple<Node, Node>> drawnGraph)
	{
		super(new FRLayout<Node, Tuple<Node,Node>>(drawnGraph));
		this.drawnGraph = drawnGraph;
		
		DefaultModalGraphMouse<Node, Tuple<Node, Node>> graphMouse = new DefaultModalGraphMouse<Node, Tuple<Node, Node>>();
		graphMouse.setMode(Mode.PICKING);
		this.setGraphMouse(graphMouse);
		
		this.getRenderContext().setVertexFillPaintTransformer(nodeColTransf=new NodeColorTransformer());
		this.getRenderContext().setEdgeDrawPaintTransformer(edgeColTransf=new EdgeColorTransformer());
		this.getRenderContext().setVertexLabelTransformer(new Transformer<Node, String>() {
			
			@Override
			public String transform(Node arg0) {
				return arg0.name;
			}
		});
		
	}
	
	void drawChanges( GraphChanger c, GraphComponent d) 
	{
		for(Node n : GraphChanger.lastAddedNodes)//add new Edges
		{
			drawnGraph.addVertex(n);
		}
		for(Tuple<Node, Node> edge : GraphChanger.lastAdded)//add new Edges
		{
			drawnGraph.addEdge(edge, edge.left, edge.right, EdgeType.UNDIRECTED);
		}

		nodeColTransf.setCurrentComp(d);
		this.repaint();
	}
}
