package visualization;

import java.util.Collection;
import java.util.HashSet;

import main.Graph;
import main.Node;
import main.SolvingState;
import main.Start;
import visualization.Graph.GraphViewing;
import Utils.Tuple;

public abstract class GraphChanger 
{
	public static HashSet<Node> lastDeletedNodes = new HashSet<Node>();
	public static HashSet<Node> lastAddedNodes = new HashSet<Node>();
	public static HashSet<Tuple<Node,Node>> lastAdded = new HashSet<Tuple<Node,Node>>();
	public static HashSet<Tuple<Node,Node>> lastDeletedEdges = new HashSet<Tuple<Node,Node>>();

	

	protected void OnBeforeChange(Graph g, GraphComponent d)
	{
		if(Start.useInitReductionVisualization)
		{
			//if(g.currentState == SolvingState.pathReductionRules&& this instanceof TreeReductionRule)
			//	return;//in this case treeReduction rules are applied after success of PRR4
			
			GraphViewing.showBeforeChange(g, this, d);
		}
	}
	
	
	public void clearChanges()
	{
		lastDeletedNodes.clear();
		lastAddedNodes.clear();
		lastAdded.clear();
		lastDeletedEdges.clear();
	}

	protected void onAfterChange(Graph g, GraphComponent d)
	{
		if(Start.useInitReductionVisualization && this.getNormalState() == g.currentState)
			GraphViewing.showAfterChange(this, d);
	}

	protected void onNodeDeleted(Node n)
	{
		if(Start.useInitReductionVisualization)
			lastDeletedNodes.add(n);	
	}
	protected void onNodesDeleted(Collection<Node> nodes)
	{
		if(Start.useInitReductionVisualization)
			lastDeletedNodes.addAll(nodes);
	}
	protected void onNodeAdded(Node n)
	{
		if(Start.useInitReductionVisualization)
			lastAddedNodes.add(n);
	}
	protected void onNodesAdded(Collection<Node> nodes)
	{
		if(Start.useInitReductionVisualization)
			lastAddedNodes.addAll(nodes);
	}
	protected void onEdgeAdded(Node n1, Node n2)
	{
		if(Start.useInitReductionVisualization)
			lastAdded.add(new Tuple<Node, Node>(n1, n2));	
	}
	
	protected void onEdgeDeleted(Node n1, Node n2)
	{
		if(Start.useInitReductionVisualization)
			lastDeletedEdges.add(new Tuple<Node, Node>(n1, n2));	
	}

	public abstract String getName();

	public boolean hasChanges() {
		return Start.useInitReductionVisualization && 
				(!lastDeletedNodes.isEmpty() || !lastDeletedEdges.isEmpty() 
						|| !lastAddedNodes.isEmpty() || !lastAdded.isEmpty());
	}


	public abstract SolvingState getNormalState();
}

